#!/usr/bin/env python3

# Copyright 2010-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Solveur de sudoku"""

import sys

from jouets.sudoku import options
from jouets.sudoku import representation
from jouets.sudoku import resolution
from jouets.sudoku.io import charge_fichier

def main():
    """Fonction principale"""

    commande = options.analyse(sys.argv[1:])
    grille = representation.Grille(charge_fichier(
        commande.fichier,
        commande.format_fichier,
        ))
    commande.traitement.execute_initial()
    resolution.resout(
        grille,
        commande.traitement.execute_courant,
        commande.processus,
        )
    commande.traitement.execute_final()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Simulation de propagation d'épidémies"""

from collections import namedtuple
from enum import Enum
from functools import total_ordering
from termcolor import colored
import blessings

from jouets.pesteetcholera import VERSION
from jouets.utils import erreurs
from jouets.utils.argparse import analyseur

TypeEtat = namedtuple('TypeEtat', ['indice', 'nom', 'couleur'])


@total_ordering
class Etat(Enum):
    """États d'un individu"""
    #pylint: disable=no-init

    vaccine = TypeEtat(1, "vacciné", "yellow")
    immunise = TypeEtat(2, "immunisé", "green")
    sain = TypeEtat(3, "sain", "blue")
    malade = TypeEtat(4, "malade", "red")
    mort = TypeEtat(5, "mort", "grey")

    @property
    def nom(self):
        """Nom de l'état."""
        return self.value.nom

    @property
    def couleur(self):
        """Couleur associées à l'état"""
        return self.value.couleur

    def __lt__(self, other):
        return self.value.indice < other.value.indice

class Terminal(blessings.Terminal):
    """Gestion de l'affichage dans le terminal"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        print()
        print()

    def affiche_population(self, population, taille_max):
        """Affiche les statistiques de la population."""
        formatter = "{{}}: {{:{}}}".format(len(str(taille_max)))
        with self.location(0, self.height-3):
            print(" | ".join([
                colored("█", etat.couleur) + formatter.format(etat.nom, nombre)
                for (etat, nombre)
                in sorted(population.items())
                ]))
        with self.location(0, self.height-2):
            print("".join([
                (colored(
                    "█", etat.couleur) * int(nombre * self.width / taille_max)
                )
                for (etat, nombre)
                in sorted(population.items())
                ]))

class PasUneProbabilite(erreurs.ErreurUtilisateur):
    """Le nombre n'est pas une probabilité."""

    def __init__(self, texte):
        super().__init__(self)
        self.texte = texte

    def __str__(self):
        return (
            "'{}' doit être une probabilité (un nombre entre 0 et "
            "1)."
            ).format(self.texte)

class TextInput:
    """Interface utilisateur en mode console"""
    #pylint: disable=no-init, too-few-public-methods

    @staticmethod
    def numinput(__titre, texte, defaut):
        """Lit un nombre.

        Continue la lecture tant qu'un nombre n'est pas fourni.
        """
        while True:
            try:
                valeur = input("{} [{}]: ".format(texte, defaut))
                if valeur == "":
                    return defaut
                return float(valeur)
            except TypeError:
                pass

def probabilite(text):
    """Renvoit la probabilité correspondant à l'argument, ou lève une exception.
    """
    try:
        proba = float(text)
    except ValueError:
        raise PasUneProbabilite(text)
    if 0 <= proba and proba <= 1:
        return proba
    else:
        raise PasUneProbabilite(text)

def analyse(parents=None):
    """Renvoit un analyseur de ligne de commande."""
    parser = analyseur(VERSION, parents=parents)

    parser.add_argument(
        '-c', '--contagion',
        metavar="PROBA",
        dest="proba_contagion",
        type=probabilite, default=None,
        help=(
            'Probability that sickness will propagate from one person to '
            'another.'
            ),
        )

    parser.add_argument(
        '-V', '--vaccine',
        metavar="PROBA",
        dest="proba_vaccin",
        type=probabilite, default=None,
        help='Probability that a particular person is vaccinated.'
        )

    parser.add_argument(
        '-H', '--heal',
        metavar="PROBA",
        dest="proba_guerison",
        type=probabilite, default=None,
        help=(
            'Probability that a sick person will heal (otherwise, he will '
            'die).'
            ),
        )

    parser.add_argument(
        '-s', '--sick',
        metavar="NUMBER",
        dest="taille_souche",
        type=int, default=None,
        help='Number of sick people in the beginning.'
        )

    parser.add_argument(
        '-p', '--population',
        metavar="NUMBER",
        dest="taille_population",
        type=int, default=None,
        help='Population size.'
        )

    parser.add_argument(
        '-n', '--neighbour',
        metavar="NUMBER",
        dest="nombre_voisins",
        type=int, default=None,
        help='Average number of neighbours of each person.'
        )

    parser.add_argument(
        '-d', '--delay',
        metavar="INT",
        dest="delai",
        type=int, default=5,
        help='Speed (the bigger the faster)'
        )

    return parser

def complete_arguments(arguments, titre, interface, defaut):
    """Complète les arguments manquants, en les demandant à l'utilisateur."""
    for nom, texte in [
            ["taille_population", "Taille de la population"],
            ["taille_souche", "Nombre initial de malades"],
            ["nombre_voisins", "Nombre de voisins"],
        ]:
        while True:
            nombre = getattr(arguments, nom)
            if nombre is not None:
                if 0 < nombre:
                    break
            try:
                setattr(
                    arguments,
                    nom,
                    int(interface.numinput(titre, texte, defaut[nom])),
                    )
            except TypeError:
                raise erreurs.Annule()

    for nom, texte in [
            ["proba_contagion", "Probabilité de contagion"],
            ["proba_guerison", "Probabilité de guérison"],
            ["proba_vaccin", "Probabilité de vaccination"],
        ]:
        while True:
            proba = getattr(arguments, nom)
            if proba is not None:
                if 0 <= proba and proba <= 1:
                    break
            try:
                setattr(
                    arguments,
                    nom,
                    float(interface.numinput(titre, texte, defaut[nom])),
                    )
            except TypeError:
                raise erreurs.Annule()

    return arguments



#!/usr/bin/env python3

# Copyright 2010-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Fonctions de resolution de sudokus.
"""

import queue
import threading

def solveur(candidats, fonction):
    """Essaye de résoudre l'ensemble des candidats.

    Boucle infinie qui prend des candidats dans la queue, tente de les
    resoudre, et applique la fonction quand ils sont resolu.
    """
    while True:
        grille = candidats.get()
        grille.remplit()
        if grille.resolu:
            fonction(grille)
        elif grille.impossible:
            pass
        else:
            (x, y) = grille.cherche_plus_proche()
            for valeur in grille.possible_case[x][y]:
                copie = grille.copy()
                copie.affecte((x, y), valeur)
                candidats.put(copie)
        candidats.task_done()


def resout(grille, fonction, nombre_processus):
    """Résout le sudoku, avec plusieurs threads.

    :arg func fonction: Fonction appellee sur les solutions trouvées.
    """
    candidats = queue.LifoQueue(0)
    candidats.put(grille)

    for __ignored in range(nombre_processus):
        thread = threading.Thread(
            target=solveur, kwargs={
                "candidats": candidats,
                "fonction": fonction,
                }
            )
        thread.daemon = True
        thread.start()

    candidats.join()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Simulation de propagation d'épidémies"""

import argparse
import collections
import contextlib
import itertools
import logging
import math
import random
import sys
import time
import turtle

from jouets.pesteetcholera.common import Etat, Terminal, TextInput
from jouets.pesteetcholera.common import analyse, complete_arguments
from jouets.utils import erreurs
from jouets.utils.argparse import yesno

LOGGER = logging.getLogger(__name__)

#pylint: disable=no-member

TITRE = "Peste"
DEFAUT = {
    "proba_guerison": .8,
    "nombre_voisins": 5,
    "proba_contagion": .5,
    "proba_vaccin": .2,
    "taille_population": 100,
    "taille_souche": 10,
}

Coordonnees = collections.namedtuple('Coordonnees', ['x', 'y'])

class FauxAffichage:
    """Classe identique à :class:`Affichage`, mais ne faisant rien.

    Utile pour désactiver l'affichage de la propagation de l'épidémie.
    """
    #pylint: disable=no-init

    _quick = 0

    def __init__(self, delai):
        self.delai = 10**(-delai/3)

    @contextlib.contextmanager
    def quick(self):
        """Passe en mode d'affichage rapide."""
        self._quick += 1
        yield
        self._quick -= 1

    def segment(self, *__args, **__kwargs):
        """Fait une pause.

        Cette méthode correspond au tracé d'un segment en mode graphique: il
        faut ici attendre pour que l'animation ne passe pas trop vite.
        """
        if not self._quick:
            time.sleep(self.delai)

    @staticmethod
    def dessine_individu(*__args, **__kwargs):
        """Dessine un individu"""
        pass

    @staticmethod
    def mainloop():
        """Boucle principale."""
        pass

class Affichage(FauxAffichage):
    """Classe gérant l'affichage de la population."""
    # pylint: disable=no-init

    def __init__(self, population, delai):
        super().__init__(delai)
        turtle.setworldcoordinates(-1, -1, population, population)
        turtle.pensize(2)
        turtle.delay(10**((9-delai)/4))

    def segment(self, individu1, individu2, etat):
        """Dessine un segment"""
        # pylint: disable=arguments-differ
        self.invisible_goto(individu1)
        self.__couleur(etat)
        turtle.goto(*individu2.coordonnees)

    def dessine_individu(self, individu):
        """Dessine un individu"""
        # pylint: disable=arguments-differ
        self.invisible_goto(individu)
        self.__couleur(individu.etat)
        turtle.dot(10)

    @contextlib.contextmanager
    def quick(self):
        """Passe momentanément en mode de dessin rapide."""
        delai = turtle.delay()
        turtle.tracer(sys.maxsize, 0)
        yield
        turtle.tracer(1, delai)

    def invisible_goto(self, individu):
        """Se déplace à un endroit sans dessiner."""
        with self.quick():
            turtle.up()
            turtle.goto(*individu.coordonnees)
            turtle.down()

    @staticmethod
    def __couleur(etat):
        """Définit la couleur en fonction de l'état."""
        turtle.color(etat.couleur)

    @staticmethod
    def mainloop():
        return turtle.mainloop()

class Population:
    """Population d'individus."""

    def __init__(self, tortue, parametres):
        self.term = Terminal()
        self.parametres = parametres
        self.tortue = tortue
        self.population = [
            [None for y in range(self.parametres.taille_population)]
            for x in range(self.parametres.taille_population)
            ]
        self.statistiques = dict([(etat, 0) for etat in Etat])
        self.statistiques[Etat.sain] = len(list(
            itertools.chain.from_iterable(self.population)
            ))

        with self.tortue.quick():
            # Création des individus
            for x in range(self.parametres.taille_population):
                for y in range(self.parametres.taille_population):
                    self.population[x][y] = Individu(self, x, y)
                    if random.random() < self.parametres.proba_vaccin:
                        self.population[x][y].etat = Etat.vaccine

            # Création des liens
            for individu in self:
                for voisin in self.proches(individu):
                    if individu not in voisin.voisins:
                        individu.ajoute_voisin(voisin)

            # Dessin des individus (les liens peuvent les avoir partiellement
            # effacés)
            for individu in itertools.chain.from_iterable(self.population):
                individu.dessine()

    def __iter__(self):
        return itertools.chain.from_iterable(self.population)

    def existe(self, x, y):
        """Renvoie ``True`` si si l'individu de coordonnées (x, y) existe.
        """
        if 0 <= x and x < self.parametres.taille_population:
            if 0 <= y and y < self.parametres.taille_population:
                return True
        return False

    def __list__(self):
        return list(itertools.chain.from_iterable(self.population))

    def proches(self, individu):
        """Renvoit les `parametres.nombre_voisins` voisins les plus proches."""
        x = individu.coordonnees.x
        y = individu.coordonnees.y
        proches = []
        for rayon in range(1, self.parametres.taille_population):
            cercle = set()
            for i in range(-rayon, rayon+1):
                for (x_voisin, y_voisin) in [
                        (x-rayon, y+i),
                        (x+rayon, y+i),
                        (x+i, y+rayon),
                        (x+i, y-rayon),
                    ]:
                    if self.existe(x_voisin, y_voisin):
                        if (
                                self.population[x_voisin][y_voisin]
                                not in individu.voisins
                            ):
                            cercle.add(self.population[x_voisin][y_voisin])
            cercle = list(cercle)
            random.shuffle(cercle)
            if len(proches) + len(cercle) <= self.parametres.nombre_voisins:
                proches.extend(cercle)
            else:
                proches.extend(
                    cercle[0:self.parametres.nombre_voisins-len(proches)]
                    )
                return proches
        return proches

    def change_etat(self, etat1, etat2):
        """Change l'état d'un individu."""
        self.statistiques[etat1] -= 1
        self.statistiques[etat2] += 1

    def propage(self, malades):
        """Propage la maladie"""
        for individu in malades:
            individu.etat = Etat.malade

        while malades:
            infecte = malades.pop()
            for voisin in infecte.voisins:
                if random.random() < self.parametres.proba_contagion:
                    if infecte.infecte(voisin):
                        malades.insert(0, voisin)
            if random.random() < self.parametres.proba_guerison:
                infecte.etat = Etat.immunise
            else:
                infecte.etat = Etat.mort
            self.term.affiche_population(
                self.statistiques,
                self.parametres.taille_population**2,
                )



class Individu:
    """Individu d'une population"""

    def __init__(self, population, x, y):
        self.coordonnees = Coordonnees(x, y)
        self._etat = Etat.sain
        self.voisins = []
        self.population = population

        self.dessine()

    def __repr__(self):
        return "Individu({}, {})".format(*self.coordonnees)

    def dessine(self):
        """Dessine l'individu"""
        self.population.tortue.dessine_individu(self)

    @property
    def etat(self):
        """Accesser de l'état"""
        return self._etat

    @etat.setter
    def etat(self, etat):
        """Mutateur de l'état"""
        self.population.change_etat(self._etat, etat)
        self._etat = etat
        self.dessine()

    def ajoute_voisin(self, voisin):
        """Ajoute un voisin à l'individu."""
        self.voisins.append(voisin)
        voisin.voisins.append(self)
        self.population.tortue.segment(
            self,
            voisin,
            Etat.sain
            )

    def infecte(self, voisin):
        """Infecte (peut-être) l'individu.

        Renvoit True si et seulement si l'individu a été infecté.
        """
        if voisin.etat == Etat.sain:
            self.population.tortue.segment(
                self,
                voisin,
                self.etat,
                )
            voisin.etat = Etat.malade
            return True
        return False

def analyse_peste():
    """Renvoit un analyseur de ligne de commande."""
    analyseur_specifique = argparse.ArgumentParser(add_help=False)
    analyseur_specifique.add_argument(
        '-t', '--turtle',
        help="Enable or disable graphical display",
        metavar="BOOLEAN",
        type=yesno,
        default=True,
        )
    return analyse(parents=[analyseur_specifique])

def main():
    """Fonction principale"""

    try:
        arguments = analyse_peste().parse_args()

        if arguments.turtle:
            arguments = complete_arguments(arguments, TITRE, turtle, DEFAUT)
            arguments.nombre_voisins //= 2
            arguments.taille_population = int(
                math.ceil(math.sqrt(arguments.taille_population))
                )
            tortue = Affichage(arguments.taille_population, arguments.delai)
        else:
            arguments = complete_arguments(arguments, TITRE, TextInput, DEFAUT)
            arguments.nombre_voisins //= 2
            arguments.taille_population = int(
                math.ceil(math.sqrt(arguments.taille_population))
                )
            tortue = FauxAffichage(arguments.delai)

        population = Population(tortue, arguments)
        population.propage(
            random.sample(list(population), arguments.taille_souche)
            )
        tortue.mainloop()
    except (erreurs.ErreurUtilisateur, KeyboardInterrupt) as erreur:
        if str(erreur):
            LOGGER.error(str(erreur))
        sys.exit(1)

    sys.exit(0)




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Génération d'un sous-jeu de base, en utilisant 'algorithme du dobble.
"""

import math

from jouets.dobble import genere_jeu as genere_dobble
from jouets.dobble.memobble import errors
from jouets.dobble.memobble import mmath

def dobble_num(num):
    """Renvoie l'argument de dobble correspondant à un jeu avec `num` symboles.
    """
    delta = 1-4*(1-num)
    if not mmath.est_carre(delta):
        raise errors.TailleNonGeree((
            "Argument '{}' must be one of `p²+p+1`, where `p` is prime."
            ).format(num))
    if not mmath.est_pair(1+int(math.sqrt(delta))):
        raise errors.TailleNonGeree((
            "Argument '{}' must be one of `p²+p+1`, where `p` is prime."
            ).format(num))
    return int((-1+math.sqrt(delta))/2)


def genere(num):
    """Génère un sous-jeu de Mémobble."""
    return genere_dobble(dobble_num(num))

ALGO = {
    'dobble': {
        'genere': genere,
        'default': {
            'num': 7,
            'sub': 4,
            }
        }
    }



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Manipulation des modules implémentant des formes de labyrinthes"""

import glob
import importlib
import logging
import os
import traceback

LOGGER = logging.getLogger(__name__)

def load_plugins(analyseur, parent, path=None, module=None):
    """Charge les plugins de formes, en enrichit ``analyseur``.

    :param analyseur: Objet retourné par un appel à
        `ArgumentParser.add_subparser()
        <https://docs.python.org/3.4/library/argparse.html#sub-commands>`_.
        Cet objet va être enrichi par les différents plugins chargés pour
        ajouter les options à la ligne de commande.
    :param parent: `Analyseur parent
        <https://docs.python.org/3.4/library/argparse.html#parents>`_, destiné
        à être inclus par les sous-commandes.
    :param path: Chemin du répertoire à inclure (sous la forme d'une chaîne
        ``str``).
    :type path: str
    :param module: Module, sous la forme d'une liste, correspondant à ``path``.
    :type module: list
    """
    if path is None:
        path = os.path.dirname(__file__)
    if module is None:
        module = ['jouets', 'labyrinthe', 'shapes']
    for nom in os.listdir(path):
        if os.path.isdir(os.path.join(path, nom)):
            if os.path.exists(os.path.join(path, nom, "__init__.py")):
                pass
            else:
                continue
        else:
            if (
                    os.path.basename(nom).endswith('.py')
                    and os.path.basename(nom) != "__init__.py"
                ):
                nom = os.path.basename(nom)[:-len('.py')]
            else:
                continue

        try:
            plugin = importlib.import_module(
                ".".join(module + [nom]),
                )
        except Exception as erreur: #pylint: disable=broad-except
            LOGGER.warning(
                "Plugin '{}' could not be loaded. Ignored:".format(
                    os.path.basename(nom)
                    )
                )
            LOGGER.warning(
                "> ".join(
                    traceback.format_exception(type(erreur), erreur, None)
                    )
                )
        else:
            plugin.complete_analyseur(analyseur, path, module, parent)



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Définitions relatives au labyrinthe utilisant le module turtle"""

#pylint: disable=no-member

import argparse
import functools
import jinja2
import logging
import os
import turtle
import pkg_resources

from jouets.labyrinthe.shapes import load_plugins
from jouets.labyrinthe.base import LabyrintheBase, MurBase, ZoneBase
from jouets.utils.listes import autre
from jouets.utils.turtle import delay

LOGGER = logging.getLogger(__name__)

def complete_analyseur(analyseur, _path, module, parent):
    "Ajoute des options à la ligne de commande, en incluant les sous-modules."
    analyseur_turtle = analyseur.add_parser(
        "turtle",
        help="Two-dimension labyrinths",
        )

    common = argparse.ArgumentParser(add_help=False)
    common.add_argument(
        '-s', '--size', default=10,
        help='Size of the labyrinth',
        )

    load_plugins(
        analyseur_turtle.add_subparsers(),
        parent + [common],
        os.path.dirname(__file__),
        module + ["turtle"],
        )

class LabyrintheTurtle(LabyrintheBase):
    """Labyrinthe en deux dimensions utilisant le module turtle

    :param fenetre: Taille de la fenêtre, sous la forme d'un ``tuple`` ``(llx,
      lly, urx, ury)`` (voir :mod:`turtle` pour la signification de ces
      coordonnées).
    :type fenetre: tuple
    :param affiche: Vaut ``True`` s'il faut dessiner le labyrinthe au fur et à
      mesure de sa création.
    :type affiche: bool
    """

    def __init__(self, fenetre, affiche):
        super().__init__()

        self.affiche = affiche
        if self.affiche:
            turtle.mode("world")
            turtle.pensize(3)
            (llx, lly, urx, ury) = fenetre
            largeur = urx - llx
            hauteur = ury - lly
            turtle.setworldcoordinates(
                llx - .1 * largeur,
                lly - .1 * hauteur,
                urx + .1 * largeur,
                ury + .1 * hauteur,
                )

    def dessine(self):
        """Dessine le labyrinthe"""
        if self.affiche:
            for mur in self.murs:
                mur.dessine()

    def construit(self):
        """Construit le labyrinthe"""
        if self.affiche:
            with delay(0):
                self.dessine()

        super().construit()

    def fin(self):
        """Attend que l'utilisateur ferme la fenêtre de ``turtle``"""
        super().fin()

        LOGGER.info("End. You can close the main window.")
        if self.affiche:
            turtle.mainloop()

    def export_tex(self, template=None):
        #pylint: disable=maybe-no-member
        if template is None:
            # Using default template
            templatename = pkg_resources.resource_filename(
                "jouets.labyrinthe",
                "data/templates/plain.tex",
                )
            loader = jinja2.FileSystemLoader(os.path.dirname(templatename))
            name = "plain.tex"
        else:
            loader = jinja2.FileSystemLoader(".")
            name = template
        return jinja2.Environment(loader=loader).get_template(name).render(
            {"murs": self.murs}
            )

class MurTurtle(MurBase):
    """Mur en deux dimensions utilisant le module turtle."""

    @property
    def extremites(self):
        """Renvoie les deux etrémités du mur.

        :rtype: tuple
        :return: extrémités, sous la forme d'un ``tuple`` de coordonnées,
          c'est-à-dire de couples de flottants.
        """
        raise NotImplementedError()

    def dessine_zones(self, color1="red", color2="orange"):
        """Dessine deux zones"""
        self.zones[0].dessine(color1)
        self.zones[1].dessine(color2)

    def export_tex(self):
        """Exporte le labyrinthe comme du code TeX."""
        raise NotImplementedError()

class MurSegment(MurTurtle):
    """Mur en forme de segment

    :param pere: Labyrinthe contenant ce mur.
    :type pere: :class:`LabyrintheBase`
    :param tuple p1: Une des extrémités du mur.
    :param tuple p2: L'autre extrémité du mur.
    """

    def __init__(self, pere, p1, p2):
        super().__init__(pere)
        #: Coordonnées du mur
        self.coordonnees = (p1, p2)

    @property
    def extremites(self):
        """Renvoie les coordonnées du mur."""
        return self.coordonnees

    def detruit(self):
        """Détruit le mur"""
        super().detruit()
        self.dessine(color="lightgray")

    def dessine(self, color="black"):
        """Dessine le mur"""
        if self.labyrinthe.affiche:
            turtle.color(color)
            turtle.up()
            turtle.goto(self.extremites[0])
            turtle.down()
            turtle.goto(self.extremites[1])

    def __str__(self):
        return "Mur({}, {})".format(
            self.extremites[0],
            self.extremites[1],
            )

    def export_tex(self):
        """Renvoie le code Tikz correspondant au mur."""
        return r"\draw ({p1[0]}, {p1[1]}) -- ({p2[0]}, {p2[1]});".format(
            p1=self.coordonnees[0],
            p2=self.coordonnees[1],
            )

@functools.total_ordering
class ZoneTurtle(ZoneBase):
    """Implémentation d'une zone plane

    Cette implémentation utilise le module :mod:`turtle`.
    """

    def iter_sommets_murs(self, sommet=None, mur=None):
        """Itérateur sur les couples ``(sommet, mur)`` de la zone.

        L'itérateur garantit que :

        * ``sommet`` est un des sommets de ``mur`` ;
        * tant que c'est possible, les murs sont énumérés de manière
          adjacentes. Ceci n'est pas toujours respecté puisqu'une zone peut
          contenir deux murs non connexes.

        :param sommet: Sommet de départ, sous la forme d'un couple de
          coordonnées (de flottants).
        :type sommet: tuple
        :param mur: Mur de départ.
        :type mur: :class:`MurTurtle`

        * Si ``mur`` est précisé, mais pas ``sommet``, un des sommets du mur
          est choisi arbitrairement comme sommet de départ.
        * Si ``sommet`` est précisé, mais pas ``mur``, un mur adjacent à ce
          sommet est choisi arbitrairement comme mur de départ.
        * Si les deux sont précisés, aucune vérification n'est faite pour
          vérifier que ce sommet est bien adjacent au mur.
        * Si aucun des deux n'est précisé, un couple (adjacent) est pris de
          manière arbitraire.

        .. warning::
            Une zone peut être constituée de deux murs non connexes (si elle
            contient entièrement une autre zone). Dans ce cas, l'itérateur
            passe d'un mur à l'autre sans prévenir.
        """
        murs = list(self.murs_inter()) + list(self.murs_exterieurs())
        if mur is None and sommet is None:
            mur = murs[0]
            sommet = mur.extremites[0]
        elif mur is None and sommet is not None:
            mur = [mur for mur in murs if sommet in mur.extremites][0]
        elif mur is not None and sommet is None:
            sommet = mur.extremites[0]
        else:
            pass
        while murs:
            suivants = [mur for mur in murs if sommet in mur.extremites]
            if suivants:
                mur = suivants[0]
                sommet = autre(mur.extremites, sommet)
            else:
                mur = murs[0]
                sommet = mur.extremites[0]
            murs.remove(mur)
            yield (sommet, mur)

    def iter_sommets(self, debut=None):
        """Itérateur sur les sommets de la zone.

        :param debut: Sommet de départ (sous la forme d'un couple de
          flottants)
        :type debut: tuple

        .. warning::
            Le même avertissement que pour :meth:`iter_sommets_murs` s'applique.
        """
        for (sommet, _mur) in self.iter_sommets_murs(sommet=debut):
            yield sommet

    def iter_murs(self, debut=None):
        """Itérateur sur les murs de la zone

        :param debut: Mur de départ.
        :type debut: :class:`MurTurtle`

        .. warning::
            Le même avertissement que pour :meth:`iter_sommets_murs` s'applique.
        """
        for (_sommet, mur) in self.iter_sommets_murs(mur=debut):
            yield mur


    def __str__(self):
        return "ZoneTurtle({})".format(
            " ".join([str(point) for point in self.iter_sommets()])
            )

    def __lt__(self, other):
        return self.aire < other.aire

    @property
    def aire(self):
        """Renvoie l'aire de la zone.

        .. warning::
            La valeur renvoyée est vraisemblablement fausse dans les cas
            suivants :

            * la zone n'est pas un polygône ;
            * la zone n'est pas connexe.
        """
        #pylint: disable=invalid-name
        somme = 0
        sommets = list(self.iter_sommets())
        (px, py) = sommets[0]
        for (x, y) in sommets[1:] + [sommets[0]]:
            somme += px * y - x * py
            (px, py) = (x, y)
        return abs(somme)



    def dessine(self, color="black"):
        """Dessine la zone

        :param color: Une couleur, telle que reconnue par le module
          :mod:`turtle`, utilisée pour dessiner la zone.
        :type color: str
        """
        if self.labyrinthe.affiche:
            for mur in self.murs_inter():
                mur.dessine(color)
            for mur in self.murs_exterieurs():
                mur.dessine(color)

    def est_valide(self):
        """Renvoie ``True`` si la zone est valide.

        La zone est valide (en plus de la validité de sa classe mère
        :class:`ZoneBase`), si les murs peuvent être correctement énumérés.
        """
        if not super().est_valide():
            return False
        try:
            list(self.iter_sommets_murs())
        except: #pylint: disable=bare-except
            return False
        return True



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""
    TODO Update comments and documentation

    sphinx.ext.todo
    ~~~~~~~~~~~~~~~

    Allow todos to be inserted into your documentation.  Inclusion of todos can
    be switched of by a configuration variable.  The todolist directive collects
    all todos of your project and lists them along with a backlink to the
    original location.

    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from docutils import nodes
from docutils.parsers.rst import directives

from sphinx.locale import _
from sphinx.environment import NoUri
from sphinx.util.nodes import set_source_info
from sphinx.util.compat import Directive, make_admonition

class proof_node(nodes.General, nodes.Element): pass
class proof_title_node(nodes.Part, nodes.Element): pass
class proof_title_name_node(nodes.Part, nodes.Element): pass
class proof_title_content_node(nodes.Part, nodes.Element): pass
class proof_content_node(nodes.General, nodes.Element): pass
#class todolist(nodes.General, nodes.Element): pass

HUMAN = {
        'property': "Propriété",
        'example': "Exemple",
        'theorem': "Théorème",
        'definition': "Définition",
        'proof': "Preuve",
        'conjecture': "Conjecture",
        'algorithm': "Algorithme",
        }


def create_environment_class(name):

    class Environment(Directive):
        """
        TODO A todo entry, displayed (if configured) in the form of an admonition.
        """

        has_content = True
        required_arguments = 0
        optional_arguments = 1
        final_argument_whitespace = True
        option_spec = {
                'label': directives.unchanged_required,
                }

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.name = name

        def run(self):
            env = self.state.document.settings.env
            targetid = 'index-%s' % env.new_serialno('index')
            targetnode = nodes.target('', '', ids=[targetid])

            node = proof_node('\n'.join(self.content))

            # Title
            title_node = proof_title_node()
            title_node['classes'] += ['proof-title']

            # Element type (property, definition, etc.)
            textnodes, messages = self.state.inline_text(HUMAN[self.name], self.lineno)
            title_name_node = proof_title_name_node(HUMAN[self.name], *textnodes)
            title_name_node['classes'] += ['proof-title-name']
            title_node += title_name_node
            node += messages

            # Element title (if relevant)
            if self.arguments:
                textnodes, messages = self.state.inline_text(self.arguments[0], self.lineno)
                title_content_node = proof_title_content_node(self.arguments[0], *textnodes)
                title_content_node['classes'] += ['proof-title-content']
                title_node += title_content_node
                node += messages

            node += title_node
            node['classes'] += ['proof', 'proof-type-{}'.format(self.name)]

            content = proof_content_node()
            self.state.nested_parse(self.content, self.content_offset, content)
            content['classes'] += ['proof-content']
            node += content

            set_source_info(self, node)
            return [targetnode, node]

    return Environment


def process_todos(app, doctree):
    # collect all todos in the environment
    # this is not done in the directive itself because it some transformations
    # must have already been run, e.g. substitutions
    env = app.builder.env
    if not hasattr(env, 'todo_all_todos'):
        env.todo_all_todos = []
    for node in doctree.traverse(proof_node):
        try:
            targetnode = node.parent[node.parent.index(node) - 1]
            if not isinstance(targetnode, nodes.target):
                raise IndexError
        except IndexError:
            targetnode = None
        newnode = node.deepcopy()
        del newnode['ids']
        env.todo_all_todos.append({
            'docname': env.docname,
            'source': node.source or env.doc2path(env.docname),
            'lineno': node.line,
            'todo': newnode,
            'target': targetnode,
        })


#class TodoList(Directive):
#    """
#    A list of all todo entries.
#    """
#
#    has_content = False
#    required_arguments = 0
#    optional_arguments = 0
#    final_argument_whitespace = False
#    option_spec = {}
#
#    def run(self):
#        # Simply insert an empty todolist node which will be replaced later
#        # when process_todo_nodes is called
#        return [todolist('')]


def process_todo_nodes(app, doctree, fromdocname):

    # Replace all todolist nodes with a list of the collected todos.
    # Augment each todo with a backlink to the original location.
    env = app.builder.env

    if not hasattr(env, 'todo_all_todos'):
        env.todo_all_todos = []

    for node in doctree.traverse(todolist):

        content = []

        for todo_info in env.todo_all_todos:
            para = nodes.paragraph(classes=['todo-source'])
            description = _('(The <<original entry>> is located in '
                            ' %s, line %d.)') % \
                          (todo_info['source'], todo_info['lineno'])
            desc1 = description[:description.find('<<')]
            desc2 = description[description.find('>>')+2:]
            para += nodes.Text(desc1, desc1)

            # Create a reference
            newnode = nodes.reference('', '', internal=True)
            innernode = nodes.emphasis(_('original entry'), _('original entry'))
            try:
                newnode['refuri'] = app.builder.get_relative_uri(
                    fromdocname, todo_info['docname'])
                newnode['refuri'] += '#' + todo_info['target']['refid']
            except NoUri:
                # ignore if no URI can be determined, e.g. for LaTeX output
                pass
            newnode.append(innernode)
            para += newnode
            para += nodes.Text(desc2, desc2)

            # (Recursively) resolve references in the todo content
            todo_entry = todo_info['todo']
            env.resolve_references(todo_entry, todo_info['docname'],
                                   app.builder)

            # Insert into the todolist
            content.append(todo_entry)
            content.append(para)

        node.replace_self(content)


def purge_todos(app, env, docname):
    if not hasattr(env, 'todo_all_todos'):
        return
    env.todo_all_todos = [todo for todo in env.todo_all_todos
                          if todo['docname'] != docname]


def visit_proof_node(self, node):
    self.body.append(self.starttag(node, 'div'))
def depart_proof_node(self, node):
    self.body.append('</div>')

def visit_proof_title_node(self, node):
    self.body.append(self.starttag(node, 'div'))
def depart_proof_title_node(self, node):
    self.body.append('</div>')

def visit_proof_title_name_node(self, node):
    self.body.append(self.starttag(node, 'span'))
def depart_proof_title_name_node(self, node):
    self.body.append('</span>')

def visit_proof_title_content_node(self, node):
    self.body.append(self.starttag(node, 'span'))
    self.body.append("(")
def depart_proof_title_content_node(self, node):
    self.body.append(")")
    self.body.append('</span>')

def visit_proof_content_node(self, node):
    self.body.append(self.starttag(node, 'div'))
def depart_proof_content_node(self, node):
    self.body.append('</div>')

def setup(app):
    app.add_javascript('proof.js')
    app.add_stylesheet('proof.css')

    #app.add_node(todolist)
    app.add_node(proof_node,
                 html=(visit_proof_node, depart_proof_node),
                 )
    app.add_node(proof_title_node,
                 html=(visit_proof_title_node, depart_proof_title_node),
                 )
    app.add_node(proof_title_name_node,
                 html=(visit_proof_title_name_node, depart_proof_title_name_node),
                 )
    app.add_node(proof_title_content_node,
                 html=(visit_proof_title_content_node, depart_proof_title_content_node),
                 )
    app.add_node(proof_content_node,
                 html=(visit_proof_content_node, depart_proof_content_node),
                 )

    app.add_directive('proof:property', create_environment_class("property"))
    app.add_directive('proof:example', create_environment_class("example"))
    app.add_directive('proof:theorem', create_environment_class("theorem"))
    app.add_directive('proof:definition', create_environment_class("definition"))
    app.add_directive('proof:proof', create_environment_class("proof"))
    app.add_directive('proof:conjecture', create_environment_class("conjecture"))
    app.add_directive('proof:algorithm', create_environment_class("algorithm"))
    #app.add_directive('todolist', TodoList)
    app.connect('doctree-read', process_todos)
    #app.connect('doctree-resolved', process_todo_nodes)
    app.connect('env-purge-doc', purge_todos)




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Format de sortie lualatex."""

import jinja2
import os
import pkg_resources
import random

SYMBOLES = [
    r"\Circle",
    r"\Square",
    r"\UParrow",
    r"\twonotes",
    r"$\spadesuit$",
    r"$\clubsuit$",
    r"\leftmoon",
    r"\ding{212}", # Flèche
    r"\ding{34}", # Ciseaux
    r"\ding{170}", # Cœur
    r"\scalebox{.7}{\smallpencil}",
    r"\scalebox{.8}{\leftthumbsup}",
    r"\scalebox{.6}{\anchor}",
    r"\scalebox{.7}{\eye}",
    r"\scalebox{.8}{\PHram}",
    r"\scalebox{.6}{\PHtunny}",
    r"\phone",
    r"\bell",
    r"\clock",
    r"\smiley",
    r"\mancube",
    r"\scalebox{.6}{\Bicycle}",
    r"\Coffeecup",
    r"\Football",
    r"\Industry",
    r"\Wheelchair",
    r"\CircledA",
    r"\Letter",
    r"\scalebox{.7}{\Bat}",
    r"\Bouquet",
    r"\Mundus",
    r"\Yinyang",
    r"\Lightning",
    r"\Radioactivity",
    r"\Stopsign",
    r"\bomb",
    r"\danger",
    r"\noway",
    r"\textxswup",
    r"\epsdice{5}",
    r"\dsagricultural",
    r"\dsheraldical",
    r"\dsjuridical",
    r"\dsliterary",
    r"\dsrailways",
    r"\dstechnical",
    r"\scalebox{.3}{\recycle}",
    r"\Info",
    r"\Gentsroom",
    r"\Ladiesroom",
    r"\MineSign",
    r"\sun",
    r"\scalebox{.5}{\gluon}",
    r"\ding{73}", # Étoile
    r"\ding{56}", # Croix
    r"\ding{100}", # Flocon de neige
    r"\ding{95}", # Fleur
    r"\dsmathematical", # Compas
    r"?",
    r"!",
    r"\#",
    r"+",
    r"\%",
    r"A",
    r"B",
    r"C",
    r"D",
    r"E",
    r"F",
    r"G",
    r"H",
    r"I",
    r"J",
    r"K",
    r"L",
    r"M",
    r"N",
    r"O",
    r"P",
    r"Q",
    r"R",
    r"S",
    r"T",
    r"U",
    r"V",
    r"W",
    r"X",
    r"Y",
    r"Z",
]

def symboles(nombre):
    """Renvoie une permutation aléatoire de `nombre`  symboles.

    Cette liste est complétée par des nombres si la liste prédéfinie de
    symboles n'est pas suffisante.
    """
    symboles_tex = SYMBOLES[:nombre]
    if len(symboles_tex) < nombre:
        symboles_tex.extend(list(range(nombre - len(symboles_tex))))
    random.shuffle(symboles_tex)
    return symboles_tex

def randomcolor(seed=None):
    """Renvoit une couleur aléatoire."""

    # On sauvegarde la graine précédente pour que l'aléatoire reste aléatoire.
    graine_precedente = random.random()

    if seed is None:
        return "0,0,0"

    random.seed(seed)
    if random.randint(1, 3) == 1:
        colors = [
            0,
            0,
            random.randint(0, 255),
            ]
    elif random.randint(2, 3) == 2:
        colors = [
            0,
            random.randint(0, 255),
            random.randint(0, 255),
            ]
    else:
        colors = [
            random.randint(127, 255),
            random.randint(0, 255),
            random.randint(0, 255),
            ]

    random.shuffle(colors)

    random.seed(graine_precedente)

    return ",".join([str(num) for num in colors])

def genere(jeu, groupe=False):
    """Renvoie le code lualatex correspondant au jeu."""
    # pylint: disable=maybe-no-member

    if groupe:
        jeux = {}
        for carte in jeu:
            if carte.groupe not in jeux:
                jeux[carte.groupe] = []
            jeux[carte.groupe].append(carte)
    else:
        jeux = {None: jeu.cartes}

    nom = "output.tex"
    return jinja2.Environment(loader=jinja2.FileSystemLoader(os.path.dirname(
        pkg_resources.resource_filename(
            "jouets.dobble.output.latex",
            os.path.join("data", "templates", nom),
            )
        ))).get_template(nom).render(
            {
                "jeux": jeux,
                "symboles": symboles(len(jeu.symboles)),
                "randint": random.randint,
                "randomcolor": randomcolor,
            }
        )

OUTPUT = {
    'tex': {
        'genere': genere,
        'description': "LuaLaTeX code",
        },
    }



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Recherche de solutions d'un problème d'optimisation."""

import argparse
import queue
import sys
import threading

from jouets.utils.argparse import analyseur

VERSION = "0.1.0"

def lignes(tableau):
    """Renvoie la représentation du tableau en chaînes, ligne par ligne.

    Ces deux tableaux sont supposés être de taille 9.
    """
    nombres = '{{:>{}}}'.format(max([len(str(x)) for x in tableau]))
    return [
        " | ".join([nombres.format(tableau[3*y+x]) for x in range(0, 3)])
        for y in range(3)
        ]

def juxtapose_tableaux(tableau1, tableau2):
    """Renvoie la chaîne correspondant à la juxtaposition de deux tableaux

    Ces deux tableaux sont supposés être de taille 9.
    """
    return "\n".join([
        "{}       {}".format(ligne1, ligne2)
        for (ligne1, ligne2) in zip(lignes(tableau1), lignes(tableau2))
        ])


class Array(object):
    """Un carré."""

    def __init__(self):
        # Dernier nombre inscrit
        self.score = 0
        # Le contenu
        self.array = [0]*9
        # Nombre de cases vides
        self.empty = 9
        # Historique du remplissage
        self.steps = []

    def fill(self, i):
        """Compléte la case i"""
        self.empty -= 1
        self.steps.append(i)
        prop = sum([self.array[3*x + y]
                    for x in range(i//3-1, i//3+2)
                    for y in range(i%3-1, i%3+2)
                    if (x >= 0 and x < 3) and (y >= 0 and y < 3)
                   ])
        if prop == 0:
            self.array[i] = 1
        else:
            self.array[i] = prop
        self.score = self.array[i]

    def __getitem__(self, i):
        return self.array[i]

    def clone(self):
        """Renvoie une copie de cet objet."""
        clone = Array()
        clone.score = self.score
        clone.array = list(self.array)
        clone.empty = self.empty
        clone.steps = list(self.steps)
        return clone

class Solution(object):
    """Carré en cours de résolution."""

    #: Dictionnaire des cases que l'on peut remplir, et des sous-classes de
    #: :class:`Solution` correspondantes.
    classes = {}

    def __init__(self, cloned=None):
        self.classes = dict([(i, Solution) for i in range(9)])
        if cloned:
            #: Grille
            self.array = cloned.array.clone()
        else:
            self.array = Array()

    def __str__(self):
        return str(self.array)

    def solve(self):
        "Renvoit la liste des objets Solution, après remplissage d'une case."
        return [
            self.classes[i](self).fill(i)
            for i in range(9)
            if ((not self.array[i]) and (i in self.classes))
            ]

    def fill(self, i):
        """Remplit la case i, et renvoit l'objet."""
        self.array.fill(i)
        return self

    @property
    def done(self):
        """Vrai si et seulement si le tableau est complètement rempli."""
        return self.array.empty == 0

    @property
    def score(self):
        """Dernière valeur inscrite dans le tableau."""
        return self.array.score

class SolutionDiagonale(Solution):
    """Solution, dont seules les cases de la diagonales sont remplies."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.classes = dict(
            [(i, SolutionDiagonale) for i in [0, 4, 8]] +
            [(i, Solution) for i in [3, 6, 7]]
            )

class SolutionVerticale(Solution):
    """Solution, dont seules les cases de la colonne centrale sont remplies."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.classes = dict(
            [(i, SolutionVerticale) for i in [1, 4, 7]] +
            [(i, Solution) for i in [0, 3, 6]]
            )

class SolutionCentrale(Solution):
    """Solution, dont seule la case centrale est remplie."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.classes = {
            0: SolutionDiagonale,
            7: SolutionVerticale,
            }

class SolutionVide(Solution):
    """Solution, vide."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.classes = {
            0: SolutionDiagonale,
            4: SolutionCentrale,
            7: SolutionVerticale,
            }



class Resolution:
    """Calcul des solutions du problème"""

    def __init__(self, nb_process=1):
        self.maximum = 0
        self.lock = threading.Lock()
        self.top = []
        self.nb_process = nb_process

        self.candidats = queue.LifoQueue(0)
        self.candidats.put(SolutionVide())

    def compare(self, solution):
        """Compare 'solution' aux meilleures solutions

        Met à jour la liste des meilleures solutions si nécessaire.
        """
        with self.lock:
            if solution.score == self.maximum:
                self.top.append(solution)
            elif solution.score > self.maximum:
                self.maximum = solution.score
                self.top = [solution]

    def worker(self):
        """Évalue les candidats, tant que cette liste n'est pas vide."""
        while True:
            grille = self.candidats.get()
            if grille.done:
                self.compare(grille)
            else:
                [self.candidats.put(child) for child in grille.solve()] # pylint: disable=expression-not-assigned
            self.candidats.task_done()

    def recherche(self):
        """Recherche les solutions au problème.

        Ne renvoie rien, mais remplit l'attribut `top`.
        """
        for __numero in range(self.nb_process):
            thread = threading.Thread(target=self.worker)
            thread.daemon = True
            thread.start()
        self.candidats.join()


    def affiche_solutions(self):
        """Recherche et affiche l'ensemble des solutions du problème."""
        self.recherche()
        solutions = []
        for item in self.top:
            ordre = [0]*9
            for i in range(9):
                ordre[item.array.steps[i]] = i
            solutions.append(juxtapose_tableaux(item.array, ordre))

        print("\n\n".join(solutions))

def _entier_naturel_non_nul(chaine):
    """Renvoie l'entier naturel non nul correspondant à la chaine.

    Lève une exception si la chaine ne représente pas un tel entier.
    """
    if float(chaine) != int(chaine):
        raise argparse.ArgumentTypeError("{} is not an integer.".format(chaine))
    if int(chaine) < 1:
        raise argparse.ArgumentTypeError(
            "{} must be greater than 1.".format(chaine)
            )
    return int(chaine)

def analyse():
    """Renvoie un analyseur de la ligne de commande."""
    parser = analyseur(
        VERSION,
        description=(
            "Solve a game (see documentation for the game description)."
            ),
        epilog=(
            "Two arrays are displayed: the resulting array, and the array of "
            "the steps used to build this resulting array."
            ),
        )
    parser.add_argument(
        '-j', '--jobs', help='Number of jobs to run simultaneously.',
        type=_entier_naturel_non_nul, default=1,
        )
    return parser

def main():
    """Fonction principale"""
    options = analyse().parse_args(sys.argv[1:])
    Resolution(options.jobs).affiche_solutions()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Création et vérification de jeux de cartes de Mémobble"""

import argparse
import functools
import itertools
import math
import os
import pluginbase
import random
import sys
import textwrap

from jouets.dobble import Carte, Jeu
from jouets.dobble.memobble import errors
from jouets.utils.argparse import analyseur
from jouets.utils.plugins import get_plugin

VERSION = "0.1.0"


def analyse():
    """Renvoie un analyseur de ligne de commande."""
    parser = analyseur(VERSION)

    parser.add_argument(
        '-n', '--num',
        type=int,
        default=None,
        help='Number of cards in each sub-game.',
        )

    parser.add_argument(
        '-s', '--sub',
        type=int,
        default=None,
        help="Number of sub-games.",
        )

    parser.add_argument(
        '-a', '--algo',
        choices=['dobble', 'algo1'],
        default='algo1',
        help="Algorithm to use to create each sub-game.",
        )
    parser.add_argument(
        '-r', '--random',
        dest='random', action='store_true',
        default=0,
        help='Random seed.',
        )
    parser.add_argument(
        '-g', '--group',
        action='store_true',
        help='Highlight connex groups of cards.',
        )

    output_plugins = get_plugin('dobble', 'output')
    parser.add_argument(
        '-f', '--format',
        choices=output_plugins.keys(),
        default='raw',
        help=(
            "Output format: {}.".format(
                ", ".join([
                    "'{}' ({})".format(keyword, value['description'])
                    for keyword, value
                    in output_plugins.items()
                    ])
            )
            ),
        )

    return parser

def genere_jeu(algo, sub, num):
    """Génère un jeu."""

    base = get_plugin('dobble.memobble', "algo")[algo]['genere'](num)

    jeu = Jeu()
    symboles = base.symboles
    for i in range(sub):
        for carte in base:
            nouvelle = Carte(groupe=i)
            for symbole in carte:
                nouvelle.symboles.append(symbole + i * len(symboles))
            jeu.cartes.append(nouvelle)

    return jeu

def process_arguments(arguments):
    """Traitement supplémentaire des arguments."""
    default = get_plugin('dobble.memobble', "algo")[arguments.algo]['default']
    if arguments.num is None:
        arguments.num = default['num']
    if arguments.sub is None:
        arguments.sub = default['sub']

    return arguments


def main():
    """Fonction principale"""
    # Argument parsing
    arguments = process_arguments(analyse().parse_args())

    random.seed(arguments.random)
    jeu = genere_jeu(arguments.algo, arguments.sub, arguments.num)

    print(get_plugin('dobble', "output")[arguments.format]['genere'](
        jeu,
        groupe=arguments.group
        ))



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Générateur itératif de fractales."""

import logging
import random
import sys
import turtle

from jouets.utils.argparse import analyseur

#pylint: disable=no-member

VERSION = "0.1.0"

LOGGER = logging.getLogger(__name__)

PREDEFINED = {
    'koch': [0, 60, -120, 60],
    }

#pylint: disable=too-few-public-methods
class _Couleur:
    """Gestion des couleurs de la fractale."""

    def __init__(self):
        self.pas = (0, 0, 0)
        self.couleur = list(random.randint(0, 255) for i in range(3))
        turtle.colormode(255)
        while self.pas == (0, 0, 0):
            self.pas = list(random.randint(-2, 2) for i in range(3))
        self.update(0)

    def update(self, compteur):
        """Met à jour la couleur si nécessaire."""
        if compteur == 0:
            self.couleur = list(self.couleur[i] + self.pas[i] for i in range(3))
            for i in range(3):
                if self.couleur[i] > 255 or self.couleur[i] < 0:
                    self.pas[i] = -self.pas[i]
                    self.couleur[i] += self.pas[i]
            turtle.color(self.couleur)

#pylint: disable=too-many-instance-attributes
class _Fenetre:
    """Gestion de la taille de la fenêtre."""

    def __init__(self):
        turtle.mode("world")
        self.llx = 0
        self.lly = 0
        self.urx = 0
        self.ury = 0
        self.f_llx = -1
        self.f_lly = -1
        self.f_urx = 1
        self.f_ury = 1
        turtle.setworldcoordinates(-1, -1, 1, 1)

    def update(self, x, y):
        """Met à jour la taille de la fenêtre si nécessaire."""
        redraw = False
        if x < self.llx:
            self.llx = x
            if x <= self.f_llx:
                self.f_llx = x
                redraw = True
        if x > self.urx:
            self.urx = x
            if x >= self.f_urx:
                self.f_urx = x
                redraw = True
        if y < self.lly:
            self.lly = y
            if y <= self.f_lly:
                self.f_lly = y
                redraw = True
        if y > self.ury:
            self.ury = y
            if y >= self.f_ury:
                self.f_ury = y
                redraw = True
        if redraw:
            turtle.setworldcoordinates(*self.new_coordinates())

    def new_coordinates(self):
        """Calcule la taille de la fenêtre."""
        (largeur_fenetre, hauteur_fenetre) = turtle.screensize()
        largeur = self.urx - self.llx
        hauteur = self.ury - self.lly
        if largeur == 0:
            self.f_llx = -((largeur_fenetre * hauteur) / hauteur_fenetre)/2
            self.f_lly = self.lly
            self.f_urx = ((largeur_fenetre * hauteur) / hauteur_fenetre)/2
            self.f_ury = self.ury
        elif hauteur == 0:
            self.f_llx = self.llx
            self.f_lly = -((hauteur_fenetre * largeur) / largeur_fenetre)/2
            self.f_urx = self.urx
            self.f_ury = ((hauteur_fenetre * largeur) / largeur_fenetre)/2
        else:
            if (largeur_fenetre / largeur) < (hauteur_fenetre / hauteur):
                nouvelle_hauteur = (hauteur_fenetre * largeur) / largeur_fenetre
                self.f_lly = self.lly - ((nouvelle_hauteur - hauteur) / 2)
                self.f_ury = self.ury + ((nouvelle_hauteur - hauteur) / 2)
                self.f_llx = self.llx
                self.f_urx = self.urx
            else:
                nouvelle_largeur = (largeur_fenetre * hauteur) / hauteur_fenetre
                self.f_llx = self.llx - ((nouvelle_largeur - largeur) / 2)
                self.f_urx = self.urx + ((nouvelle_largeur - largeur) / 2)
                self.f_lly = self.lly
                self.f_ury = self.ury

        largeur_fenetre = self.f_urx - self.f_llx
        hauteur_fenetre = self.f_ury - self.f_lly
        self.f_llx -= largeur_fenetre * 0.1
        self.f_lly -= hauteur_fenetre * 0.1
        self.f_urx += largeur_fenetre * 0.1
        self.f_ury += hauteur_fenetre * 0.1
        return self.f_llx, self.f_lly, self.f_urx, self.f_ury

class Fractale:
    """Tracé de fractale"""

    #: Liste des étapes courantes de chaque profondeur de motifs
    compteur = []
    #: Nombre d'angles dans le motif de base
    base = 0
    #: Liste des angles pris par la tortue lors du parcours du motif de base
    angles = []

    def __init__(self, angles):
        self.compteur = [-1]

        self.angles = list(angles)
        self.base = len(angles)
        self.angles.append(-sum(angles))

    def __iter__(self):
        """Itérateur sur les angles à prendre pour tracer la fractale"""
        while True:
            index = 0
            retenue = 1
            angle = 0
            while retenue != 0:
                self.compteur[index] += 1
                angle += self.angles[self.compteur[index]]
                if self.compteur[index] == self.base:
                    self.compteur[index] = 0
                    if index == (len(self.compteur) - 1):
                        self.compteur.append(0)
                    angle += self.angles[self.compteur[index]]
                    index += 1
                    retenue = 1
                else:
                    retenue = 0
            yield angle

def trace(angles, rapide):
    """Trace la fractale

    :param list angles: Liste des angles pris par la tortue.
    :param bool rapide: Tracé rapide ou non.
    """
    # Initialisation
    fractale = Fractale(angles)
    fenetre = _Fenetre()
    couleur = _Couleur()
    if rapide:
        turtle.tracer(fractale.base, 0)
    turtle.title(
        "Fractale — {}".format(" ".join([str(angle) for angle in angles]))
        )

    # Tracé
    for angle in fractale:
        turtle.left(angle)
        turtle.forward(1)
        fenetre.update(turtle.xcor(), turtle.ycor())
        couleur.update(fractale.compteur[0])

def analyse():
    """Renvoie un analyseur de ligne de commande"""
    parser = analyseur(VERSION)

    parser.add_argument(
        '-f', '--fast',
        action='store_true',
        help="Fast drawing."
        )

    parser.add_argument(
        '-t', '--type',
        metavar="NAME",
        action='store',
        choices=PREDEFINED.keys(),
        help="Predefined fractals: 'koch': Koch snowflake,"
        )

    parser.add_argument(
        'angles',
        metavar='angle',
        type=int,
        nargs='*',
        help="Angles of the base drawing of the frarctal.",
        )

    return parser

def main():
    """Fonction principale"""
    arguments = analyse().parse_args(sys.argv[1:])

    angles = arguments.angles
    if arguments.type:
        angles = PREDEFINED[arguments.type]
    if len(angles) < 2:
        LOGGER.error("Please provide at least two angles.")
        return 1

    try:
        trace(angles, rapide=arguments.fast)
    except (KeyboardInterrupt, turtle.TurtleGraphicsError, turtle.TK.TclError):
        return 0



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de egyptienne.py"""

#pylint: disable=invalid-name

import unittest

from jouets.egyptienne import egyptienne, irreductible, pgcd

# pylint: disable=too-many-public-methods
class TestFonctions(unittest.TestCase):
    """Vérifications des fonctions"""

    def test_pgcd(self):
        """Vérification de la fonction pgcd()"""
        temoin = [
            [2, 2, 2],
            [3, 5, 1],
            [15, 35, 5],
            [150, 60, 30],
            ]
        for (a, b, resultat) in temoin:
            self.assertEqual(
                resultat,
                pgcd(a, b),
            )

    def test_irreductible(self):
        """Vérification de la fonction irreductible()"""
        temoin = [
            [2, 3, (2, 3)],
            [10, 15, (2, 3)],
            ]
        for (a, b, resultat) in temoin:
            self.assertTupleEqual(
                resultat,
                irreductible(a, b),
            )

    def test_egyptienne(self):
        """Vérification de la fonction egyptienne()"""
        temoin = [
            [1, 3, [3]],
            [2, 3, [2, 6]],
            [9, 20, [3, 9, 180]],
            [4, 3, [1, 3]],
            ]
        for (a, b, resultat) in temoin:
            self.assertListEqual(
                resultat,
                egyptienne(a, b),
            )

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Simulation de propagation d'épidémies"""

import argparse
import time
import random

from jouets.pesteetcholera.common import Etat, Terminal, TextInput
from jouets.pesteetcholera.common import analyse, complete_arguments

TITRE = "Choléra"
DEFAUT = {
    "nombre_voisins": 5,
    "proba_contagion": .5,
    "proba_guerison": .8,
    "proba_vaccin": .2,
    "taille_population": 1000,
    "taille_souche": 10,
}

class Population:
    """Population d'individus"""

    def __init__(self, parametres):

        # Définition des paramètres
        self.taille = parametres.taille_population
        self.nombre_voisins = parametres.nombre_voisins
        self.proba_contagion = parametres.proba_contagion
        self.proba_guerison = parametres.proba_guerison
        self.delai = 10**(-parametres.delai/3)

        # Initialisation de la population
        self.population = dict()
        self.population[Etat.malade] = parametres.taille_souche
        self.population[Etat.vaccine] = int(
            parametres.proba_vaccin * self.taille
            )
        self.population[Etat.immunise] = 0
        self.population[Etat.mort] = 0
        self.population[Etat.sain] = (
            self.taille - sum(self.population.values())
            )

    def change_etat(self, etat1, etat2):
        """Change l'état d'un individu"""
        self.population[etat1] -= 1
        self.population[etat2] += 1

    def individu_aleatoire(self):
        """Renvoit l'état d'un individu tiré au hasard dans la population."""
        alea = random.randrange(self.taille)
        for (etat, nombre) in self.population.items():
            if alea < nombre:
                return etat
            alea -= nombre

    def propage(self):
        """Propage l'épidémie"""
        term = Terminal()
        while self.population[Etat.malade]:
            for __ignored in range(self.nombre_voisins):
                voisin = self.individu_aleatoire()
                if (
                        voisin == Etat.sain
                        and
                        random.random() < self.proba_contagion
                    ):
                    self.change_etat(voisin, Etat.malade)
            if random.random() < self.proba_guerison:
                self.change_etat(Etat.malade, Etat.immunise)
            else:
                self.change_etat(Etat.malade, Etat.mort)
            time.sleep(self.delai)
            term.affiche_population(self.population, self.taille)

def analyse_cholera():
    """Renvoit un analyseur de ligne de commande."""
    return analyse(parents=[argparse.ArgumentParser(add_help=False)])

def main():
    """Fonction principale"""
    arguments = complete_arguments(
        analyse_cholera().parse_args(),
        TITRE,
        TextInput,
        DEFAUT,
        )
    Population(arguments).propage()




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Facilite la définition et l'utilisation de plugins."""

import importlib
import os
import pluginbase

def basedir(base):
    """Renvoit le répertoire correspondant au module `base` (relatif à jouets).
    """
    return os.path.dirname(
        importlib.import_module(
            'jouets.{}'.format(base)
            ).__file__
        )

def get_plugin(base, path):
    """Renvoit le dictionnaire de plugins présents dans le chemin ``path``.

    Un plugin est un module contenant un attribut
    ``PATH`` (``path`` en lettres majuscules), de
    type dictionnaire (:type:`dict`).
    """
    plugins = pluginbase.PluginBase(
        package='jouets.{}.{}'.format(base, path),
        )
    plugins = plugins.make_plugin_source(
        searchpath=[os.path.join(
            basedir(base),
            path,
            )],
        persist=True,
        )
    plugin_dict = {}
    for name in plugins.list_plugins():
        plugin = plugins.load_plugin(name)
        try:
            plugin_dict.update(getattr(plugin, path.upper()))
        except KeyError:
            continue
    return plugin_dict




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de labyrinthe.py"""

import unittest

from jouets.labyrinthe.shapes.turtle.square import \
        Labyrinthe as LabyrintheCarre
from jouets.labyrinthe.shapes.turtle.triangle import \
        Labyrinthe as LabyrintheTriangle
from jouets.labyrinthe.shapes.turtle.random1 import \
        Labyrinthe as LabyrintheRandom1

# pylint: disable=too-many-public-methods
class TestPremiers(unittest.TestCase):
    """Vérification de la liste des nombres générés"""

    def test_carre(self):
        """Génération et vérification d'un labyrinthe à base carrée"""
        lab = LabyrintheCarre(taille=10, affiche=False)
        lab.construit()
        self.assertListEqual(
            lab.invalides(),
            [],
        )

    def test_triangle(self):
        """Génération et vérification d'un labyrinthe à base triangulaire"""
        lab = LabyrintheTriangle(taille=10, affiche=False)
        lab.construit()
        self.assertListEqual(
            lab.invalides(),
            [],
        )

    def test_random1(self):
        """Génération et vérification d'un labyrinthe à base aléatoire"""
        lab = LabyrintheRandom1(taille=10, affiche=False)
        lab.construit()
        self.assertListEqual(
            lab.invalides(),
            [],
        )

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Crible d'Érathostène optimisé en mémoire"""

import sys

from jouets.utils.argparse import analyseur
from jouets.utils.listes import ListeTriee

VERSION = "0.1.0"

def premiers():
    """Itérateur des nombres premiers"""
    prochains = ListeTriee()
    yield 2
    yield 3
    prochains.push(3, 9)
    nombre = 5
    while True:
        premier, multiple = prochains.pop()
        while multiple != nombre:
            yield nombre
            prochains.push(nombre, nombre**2)
            nombre += 2
        nombre += 2
        prochains.push(premier, prochains.suivant(multiple, 2*premier))

def est_premier(nombre):
    """Renvoie ``True`` si le nombre donné en argument est premier."""
    for i in premiers():
        if i < nombre:
            continue
        if i == nombre:
            return True
        if i > nombre:
            return False

def _affiche_premiers():
    """Affiche les nombres premiers."""
    try:
        for premier in premiers():
            print(premier)
    except KeyboardInterrupt:
        sys.exit(0)
    except BrokenPipeError:
        sys.exit(0)

def analyse():
    """Renvoie un analyseur de la ligne de commande."""
    return analyseur(
        VERSION,
        description="Print primes numbers",
        epilog="This program prints prime numbers until it is killed.",
        )

def main():
    """Fonction principale"""
    analyse().parse_args(sys.argv[1:])
    _affiche_premiers()



#!/usr/bin/env python3

# Copyright 2012-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Analyse de la ligne de commande."""

import argparse
import sys
import textwrap
import threading
import time

from jouets.sudoku.version import VERSION
from jouets.utils.argparse import analyseur as analyseur_base

def analyseur():
    """Renvoit un analyseur de ligne de commandes."""
    parser = analyseur_base(
        VERSION,
        description="Un solveur de Sudoku",
    )
    parser.add_argument(
        'fichier', nargs='?', type=argparse.FileType('r'),
        default=sys.stdin, help=textwrap.dedent("""\
            File containing the starting array. If absent, arry is read from
            standard input.
            """),
        )
    parser.add_argument(
        '-f', '--format', action='store',
        dest='format_fichier', type=str, choices=["short", "long"],
        default="long", help=textwrap.dedent("""\
            Input file format: "short" (...232....4....1) or long (lines of ". . . 2 3").
            """),
        )
    parser.add_argument(
        '-j', '--jobs', action='store',
        dest='processus', type=int, default=1,
        help="Number of jobs to run simultaneously.",
        )
    parser.add_argument(
        '-a', '--actions', action='store', dest='actions',
        type=str, default='p',
        help=textwrap.dedent("""\
            Action to perform on solved arrays. This argument is a combination
            of letters 'p' (print solution), 'c' (count solutions), and 't'
            (time solutions).
            """),
        )
    return parser


def analyse(ligne_de_commande):
    """Analyse la ligne de commande, et renvoie un namedtuple:

    - traitement : liste de ce qui doit etre effectue sur les solutions (objet
      de type Action).
    - format_fichier : "short" ou "long"
    - fichier : fichier (deja ouvert) dans lequel lire la grille (entree
      standart autorisee).
    - processus : nombre de processus concurrents a utiliser.
    """
    options = analyseur().parse_args(ligne_de_commande)

    options.traitement = Action()
    for char in options.actions:
        if char == "c":
            options.traitement.compte(True)
        elif char == "t":
            options.traitement.chronometre(True)
        elif char == "p":
            options.traitement.affiche(True)
        else:
            raise Exception(
                "Letter {} not recognized as an argument to {}.".format(
                    char,
                    "--actions",
                    )
                )
    del options.actions

    return options

class Action:
    """Gestion des actions a effectuer avant, pendant et apres la resolution.
    """
    # Ensemble des fonctions a executer avant la resolution. Ces fonctions ne
    # prennent pas d'argemnt.
    __actions_initiales = set()
    # Ensemble des fonctions a executer pour chaque grille resolue. Ces
    # fonctions prennent un argement : la grille en question.
    __actions_courantes = set()
    # Ensemble des fonctions a executer apres la resolution. Ces fonctions ne
    # prennent pas d'argemnt.
    __actions_finales = set()

    # Nombre de grilles resolues
    __compte_valeur = 0

    # Heure de depart
    __chronometre_depart = 0

    # Semaphore pour que deux processus n'affichent pas le resultat en meme
    # temps.
    __affiche_semaphore = threading.Semaphore(1)

    def __init__(self):
        pass

    @staticmethod
    def __definit(ensemble, fonction, oui):
        """Ajoute ou enleve une fonction d'un ensemble

        :arg set ensemble: Ensemble à manipuler
        :arg func  fonction: Fonction à manipuler
        :arg bool oui: Si `True`, ajoute la fonction; sinon, l'enlève.
        """
        if oui:
            ensemble.add(fonction)
        else:
            ensemble.discard(fonction)

    def execute_initial(self):
        """Execute l'ensemble des fonctions initiales
        """
        for fonction in self.__actions_initiales:
            fonction()

    def execute_courant(self, argument):
        """Execute l'ensemble des fonctions a lancer pour chaque grille trouvee
        """
        for fonction in self.__actions_courantes:
            fonction(argument)

    def execute_final(self):
        """Execute l'ensemble des fonctions finales
        """
        for fonction in self.__actions_finales:
            fonction()

    # Compte
    def compte(self, oui):
        """Definit s'il faut compter le nombre de solutions trouvees.
        """
        self.__definit(self.__actions_courantes, self.__compte_incremente, oui)
        self.__definit(self.__actions_finales, self.__compte_affiche, oui)

    def __compte_incremente(self, __ignore__):
        """Augmente le nombre de grilles trouvees
        """
        self.__compte_valeur += 1

    def __compte_affiche(self):
        """Affiche le nombre de grilles trouvees
        """
        print("%s solutions\n" % self.__compte_valeur)

    # Affiche
    def affiche(self, oui):
        """Definit si oui ou non il faut afficher les grilles trouvees
        """
        self.__definit(self.__actions_courantes, self.__affiche, oui)

    def __affiche(self, argument):
        """Affiche la grille
        """
        self.__affiche_semaphore.acquire()
        print(argument.pretty_print())
        self.__affiche_semaphore.release()

    # Chronometre
    def chronometre(self, oui):
        """Definit si oui ou non il faut chronometrer le temps de resolution
        """
        self.__definit(self.__actions_initiales, self.__chronometre_debut, oui)
        self.__definit(self.__actions_finales, self.__chronometre_fin, oui)

    def __chronometre_debut(self):
        """Enregistre l'heure de depart
        """
        self.__chronometre_depart = time.clock()

    def __chronometre_fin(self):
        """Affiche le temps mis.
        """
        print(
            '%.2f secondes' %
            float(time.clock() - self.__chronometre_depart)
            )




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Génération d'un sous-jeu de base, en utilisant 'algorithme 1.
"""

import itertools

from jouets.dobble import Carte, Jeu
from jouets.dobble.memobble import errors
from jouets.dobble.memobble.mmath import est_pair

def genere(num):
    """Genère un sous-jeu, en utilisant l'algorithme 1.

    """
    if not est_pair(num):
        return errors.TailleNonGeree("Argument '{}' must be even.".format(num))
    symboles = itertools.count()
    cartes1 = [Carte() for i in range(num//2)]
    cartes2 = [Carte() for i in range(num//2)]
    for i in cartes1:
        for j in cartes2:
            symbole = next(symboles)
            i.symboles.append(symbole)
            j.symboles.append(symbole)
    return Jeu(cartes1 + cartes2)


ALGO = {
    'algo1': {
        'genere': genere,
        'default': {
            'num': 6,
            'sub': 4,
            }
        }
    }



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Tests"""

import doctest
import os
import unittest

import jouets

def suite():
    """Renvoie un objet TestSuite, pour tester l'ensemble de `jouets`.

    Both unittest and doctest are tested.
    """
    test_loader = unittest.defaultTestLoader
    return test_loader.discover(os.path.dirname(__file__))

def load_tests(__loader, tests, __pattern):
    """Load tests (unittests and doctests)."""
    # Loading doctests
    tests.addTests(doctest.DocTestSuite(jouets))

    # Unittests are loaded by default
    return tests

if __name__ == "__main__":
    unittest.TextTestRunner().run(suite())



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Labyrinthe à base d'un maillage de triangles."""

from math import sqrt
import itertools

from jouets.labyrinthe.shapes.turtle import \
        LabyrintheTurtle, MurSegment, ZoneTurtle

def complete_analyseur(analyseur, _path, _module, parent):
    """Ajoute des options à la ligne de commande."""
    analyseur_triangle = analyseur.add_parser(
        "triangle",
        help="Triangle basic structure",
        description="Two dimension labyrinth with a triangle basic structure.",
        parents=parent,
        )
    analyseur_triangle.set_defaults(labyrinthe=Labyrinthe)

class Labyrinthe(LabyrintheTurtle):
    """Implémentation d'un labyrinthe à base d'un maillage carré.

    :param int taille: Taille du maillage
    :param bool affiche: Même signification que pour
        :class:`~jouets.labyrinthe.shapes.turtle.LabyrintheTurtle`.
    """

    def __init__(self, taille, affiche):
        super().__init__((0, 0, taille, taille * sqrt(3)/2), affiche)

        zones_gauche = [
            [ZoneTurtle(self) for y in range(taille - x)]
            for x in range(taille)
            ]
        zones_droite = [
            [ZoneTurtle(self) for y in range(taille - x - 1)]
            for x in range(taille - 1)
            ]
        murs_bas = [
            [MurSegment(
                self, (x + .5*y, y*sqrt(3)/2), (x+1 + .5*y, y*sqrt(3)/2)
                )
             for y in range(taille - x)]
            for x in range(taille)
            ]
        murs_montant = [
            [MurSegment(
                self, (x + .5*y, y*sqrt(3)/2), (x+.5*y+.5, (y+1)*sqrt(3)/2)
                )
             for y in range(taille - x)]
            for x in range(taille)
            ]
        murs_descendant = [
            [MurSegment(
                self, (x + .5*y+.5, (y+1)*sqrt(3)/2), (x+1+.5*y, y*sqrt(3)/2)
                )
             for y in range(taille - x)]
            for x in range(taille)
            ]

        #pylint: disable=invalid-name
        for x in range(taille):
            for y in range(taille - x):
                murs_bas[x][y].affecte_zone(zones_gauche[x][y])
                murs_montant[x][y].affecte_zone(zones_gauche[x][y])
                murs_descendant[x][y].affecte_zone(zones_gauche[x][y])
                if x + y != taille - 1:
                    murs_bas[x][y+1].affecte_zone(zones_droite[x][y])
                    murs_montant[x+1][y].affecte_zone(zones_droite[x][y])
                    murs_descendant[x][y].affecte_zone(zones_droite[x][y])

        self.murs = list(itertools.chain.from_iterable(
            murs_bas + murs_montant + murs_descendant
            ))
        self.zones = list(itertools.chain.from_iterable(
            zones_gauche + zones_droite
            ))



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Définition de classes abstraites de manipulation de labyrinthes."""

import random

from jouets.utils.erreurs import ErreurInterne

class LabyrintheBase:
    """Classe abstraite représentant un labyrinthe"""

    def __init__(self):
        #: Ensemble des murs
        self.murs = set()
        #: Ensemble des zones
        self.zones = set()

    def murs_intra(self):
        """Itérateur sur les murs intérieurs à des zones."""
        for mur in self.murs:
            if mur.est_intra():
                yield mur

    def murs_inter(self):
        """Itérateur sur les murs entre différentes zones."""
        for mur in self.murs:
            if mur.est_inter():
                yield mur

    def murs_exterieurs(self):
        """Itérateur sur les murs extérieurs."""
        for mur in self.murs:
            if mur.est_exterieur():
                yield mur

    def construit(self):
        """Détruit des murs jusqu'à obtention d'un labyrinthe connexe."""
        while list(self.murs_inter()):
            mur = random.choice(list(self.murs_inter()))
            mur.detruit()

    def fin(self):
        """Fonction appelée lorsque le labyrinthe a été généré."""
        pass

    def invalides(self):
        """Renvoie la liste des éléments du labyrinthe invalides.

        :returns: Liste des éléments (murs et zones) invalides.
        :rtype: ``list``

        Cette fonction est utilisée pour les tests.
        """
        a_traiter = list(self.murs) + list(self.zones)
        faits = []
        invalides = []
        while a_traiter:
            element = a_traiter.pop()
            faits.append(element)
            if not element.est_valide():
                invalides.append(element)
            if isinstance(element, MurBase):
                for zone in element.zones:
                    if (zone is not None) and (zone not in faits):
                        a_traiter.append(zone)
            elif isinstance(element, ZoneBase):
                for mur in element.murs:
                    if mur not in faits:
                        a_traiter.append(mur)
        return invalides

    def export_tex(self, template=None):
        """Exporte le labyrinthe comme du code TeX."""
        raise NotImplementedError()

class MurBase:
    """Classe abstraite représentant un mur.

    :param pere: Labyrinthe dans lequel est situé le mur.
    :type pere: :class:`LabyrintheBase`
    """

    def __init__(self, pere):
        #: Zones adjacentes à ce mur. Une zone peut-être égale à ``None`` si le
        #: mur est un mur extérieur.
        self.zones = [None, None]
        #: Labyrinthe dans lequel est situé le mur.
        self.labyrinthe = pere
        self.labyrinthe.murs.add(self)

    def detruit(self):
        """Détruit le mur, et met à jour les zones adjacentes.

        Précondition : le mur est un mur entre deux zones différentes.
        """
        petite, grande = sorted(self.zones)
        grande.absorbe(petite)
        grande.murs.remove(self)
        self.labyrinthe.murs.remove(self)

    def change_zone(self, origine, destination):
        """Met à jour les zones d'un mur.

        :param origine: Zone à supprimer.
        :param destination: Zone par laquelle remplacer ``origine``.
        :type origine: :class:`ZoneBase`
        :type destination: :class:`ZoneBase`

        Remplace ``origine`` par ``destination``, en mettant à jour les
        attribut des zones concernées.
        """
        destination.murs.add(self)
        origine.murs.remove(self)
        for i in range(2):
            if self.zones[i] == origine:
                self.zones[i] = destination

    def affecte_zone(self, zone):
        """Attribue une zone à ce mur.

        :param zone: Zone à attribuer.
        :type zone: :class:`ZoneBase`.
        """
        if (zone is None) and (None in self.zones):
            return
        for i in range(2):
            if self.zones[i] is None:
                self.zones[i] = zone
                zone.murs.add(self)
                return
        raise ErreurInterne()

    def est_inter(self):
        """Renvoie ``True`` si ce mur a deux zones adjacentes"""
        return self.zones[0] != self.zones[1] and not self.est_exterieur()

    def est_intra(self):
        """Renvoie ``True`` si ce mur est interne à une zone"""
        return self.zones[0] == self.zones[1] and not self.est_exterieur()

    def est_exterieur(self):
        """Renvoie ``True`` si ce mur est extérieur"""
        return None in self.zones

    def est_valide(self):
        """Renvoie ``True`` si ce mur est valide.

        Un mur valide est référencé par ses zones adjacentes, et a au moins une
        zone adjacente.
        """
        if len(self.zones) != 2:
            return False
        for zone in self.zones:
            if zone is not None:
                if self not in zone.murs:
                    return False
        return True

class ZoneBase:
    """Représentation d'une zone d'un labyrinthe.

    Une zone est un ensemble connexe du labyrinthe.

    :param pere: Labyrinthe dans lequel est situé le mur.
    :type pere: :class:`LabyrintheBase`
    """

    def __init__(self, pere):
        #: Ensemble des murs (intra et inter zones) adjacents à la zone.
        self.murs = set()
        #: Labyrinthe dans lequel cette zone est située.
        self.labyrinthe = pere
        self.labyrinthe.zones.add(self)

    def absorbe(self, zone):
        """Absorbe la zone ``zone``.

        Cette zone récupère les murs de ``zone``, et cette dernière est
        supprimée.
        """
        copie_murs = zone.murs.copy()
        for mur in copie_murs:
            mur.change_zone(zone, self)
        self.labyrinthe.zones.remove(zone)

    def murs_intra(self):
        """Itérateur sur les murs internes de cette zone."""
        for mur in self.murs:
            if mur.est_intra():
                yield mur

    def murs_inter(self):
        """Itérateur sur les murs de cette zone adjacents à une autre zone."""
        for mur in self.murs:
            if mur.est_inter():
                yield mur

    def murs_exterieurs(self):
        """Itérateur sur les murs extérieurs de cette zone."""
        for mur in self.murs:
            if mur.est_exterieur():
                yield mur

    def est_valide(self):
        """Renvoie ``True`` si cette zone est valide.

        Une zone est valide si elle est référencée par ``self.labyrinthe``, et
        par chacun de ses murs.
        """
        for mur in self.murs:
            if self not in mur.zones:
                return False
        if self not in self.labyrinthe.zones:
            return False
        return True



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

#pylint: disable=invalid-name

"""Décomposition en fractions égyptiennes."""

import sys

from jouets.utils.argparse import analyseur

VERSION = "0.1.0"

def pgcd(a, b):
    """Renvoie le PGCD de ``a`` et ``b``

    L'algorithme mis en œuvre est celui d'`Euclide
    <http://fr.wikipedia.org/wiki/Algorithme_d%27Euclide>`_.
    """
    if a < b:
        return pgcd(b, a)
    r = a % b
    if r == 0:
        return b
    return pgcd(b, r)

def irreductible(a, b):
    """Simplifie la fraction ``a/b``.

    Renvoie un couple ``(numérateur, dénominateur)``.
    """
    n = pgcd(a, b)
    return (a//n, b//n)

def egyptienne(a, b):
    """Décompose ``a/b`` en fraction égyptienne.

    Renvoie la liste des dénominateurs de la décomposition.
    """
    (a, b) = irreductible(a, b)

    if a == 1:
        return [b]

    (n, r) = divmod(b, a)
    return [n+1] + egyptienne(
        a * (a - r),
        b * (b - r + a),
        )

def _print_egyptienne(a, b, denominateurs):
    """Affiche la fraction et sa décomposition."""
    print("{}/{} = {}".format(
        a,
        b,
        " + ".join(
            ["1/{}".format(n) for n in denominateurs]
        ),
        ))

def analyse():
    """Renvoie un analyseur de la ligne de commande."""
    parser = analyseur(
        VERSION,
        description="Calculate egyptian fractions",
        )
    parser.add_argument(
        'fraction', metavar='A/B', nargs='+', type=str,
        help=(
            "Fractions to decompose, in the form 'a/b' (where a and b are "
            "integers)."
        )
    )
    return parser

class _ErreurOptions(Exception):
    """Erreurs dans les options de la ligne de commande."""

    def __init__(self, fraction):
        super().__init__(self)
        self.fraction = fraction

    def __str__(self):
        return (
            "Error: Fraction '{}' is not of the form INT/INT (where the "
            "second int is not zero)."
            ).format(self.fraction)

def _analyse_fraction(texte):
    """Analyse une fraction, telle qu'entrée dans la ligne de commandes.

    Le texte doit être de la forme *INT/INT*, où *INT* sont des entiers, le
    second étant non nul.

    Renvoie le couple ``(numérateur, dénominateur)`` correspondant à la
    fraction.

    Lève une exception *_ErreurOptions* si le format attendu n'est pas
    respecté.
    """
    liste = texte.split('/')
    if len(liste) != 2:
        raise _ErreurOptions(texte)

    try:
        (a, b) = [int(n) for n in liste]
    except ValueError:
        raise _ErreurOptions(texte)

    if b == 0:
        raise _ErreurOptions(texte)

    return (a, b)

def main():
    """Fonction principale

    Prend en argument les arguments de la ligne de commande.
    """
    for fraction in analyse().parse_args(sys.argv[1:]).fraction:
        try:
            (a, b) = _analyse_fraction(fraction)
        except _ErreurOptions as erreur:
            sys.stderr.write(str(erreur) + "\n")
            continue

        _print_egyptienne(
            a, b,
            egyptienne(a, b)
            )



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Création de labyrinthes"""

import argparse
import logging
import random
import sys

from jouets.labyrinthe import shapes
from jouets.utils.argparse import yesno, analyseur
from jouets.utils.erreurs import ErreurInterne

LOGGER = logging.getLogger(__name__)

VERSION = "0.1.0"

def analyse():
    """Renvoie un analyseur de la ligne de commande."""
    parser = analyseur(
        VERSION,
        description="Generate labyrinths",
        )

    common = argparse.ArgumentParser(add_help=False)
    common.add_argument(
        '--format', '-f',
        choices=['tex'],
        default=None,
        help='Output format',
        )
    common.add_argument(
        '--template', '-t',
        default=None,
        help='Template to use for the LaTeX output format.',
        )
    common.add_argument(
        '--display', '-d',
        choices=['yes', 'no'],
        default='yes',
        help='Display labyrinth building'
        )

    analyseur_formes = parser.add_subparsers(
        title="shapes",
        description="Subcommands providing shapes",
        )
    shapes.load_plugins(analyseur_formes, parent=[common])

    return parser

def main():
    """Fonction principale"""
    options = analyse().parse_args(sys.argv[1:])

    if 'labyrinthe' not in options:
        LOGGER.error('(ERROR) Please choose a labyrinth.')
        sys.exit(1)

    lab = options.labyrinthe(
        taille=int(options.size),
        affiche=yesno(options.display),
        )
    lab.construit()
    if options.format == "tex":
        print(lab.export_tex(options.template))
    lab.fin()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Résolution (naïve) du problème des apéritifs."""

import os
import sys

from jouets.utils.argparse import analyseur

VERSION = "0.1.0"

def _lire_conf():
    """Lit le fichier de configuration.

    Renvoie, en centimes, la liste des prix définis dans le fichier
    'aperitif.txt'. Renvoie une liste vide si ce fichier n'existe pas.
    """
    try:
        prix = []
        with open(os.path.join(
            os.path.dirname(__file__),
            'aperitif.txt',
            ), 'r') as fichier:
            for ligne in fichier:
                if ligne.strip().startswith('#'):
                    continue
                prix.extend(ligne.replace(',', '.').strip('\n').split(' '))
        return sorted(list(set([
            int(100 * float(item))
            for item
            in prix
            if item
            ])))
    except FileNotFoundError:
        return []

def _lire_prix():
    """Lit la liste des prix.

    Si un tel fichier existe, propose le contenu de 'aperitif.txt' comme
    valeurs par defaut.

    Renvoie, en centimes, la liste des prix.
    """
    try:
        fichier = _lire_conf()
        message = "Liste des prix, separes par des espaces"
        if fichier:
            message += " (laisser vide pour {})".format(", ".join([
                str(float(item)/100)
                for item
                in fichier
                ]))
        message += " ? "
        prix = sorted(list(set([
            int(100 * float(valeur.replace(',', '.')))
            for valeur
            in input(message).split(' ')
            if valeur
            ])), reverse=True)
        if prix:
            return prix
        else:
            return fichier
    except EOFError:
        print()
        return None

def _lire_total():
    """Lit le prix total."""
    print(20*"=")
    try:
        message = "Total (laisser vide pour quitter) ? "
        entree = input(message).replace(',', '.')
    except EOFError:
        print()
        return None
    if not entree.strip():
        return None
    return int(100*float(entree))

def aperitif(total, prix):
    """Iterateur renvoyant les solutions du problème des apéritifs.

    :arg total int: Prix total à atteindre.
    :arg prix list: Liste des prix disponibles.

    :return: Itérateur sur la liste des solutions, ces solutions étant des
             listes correspondant à ``prix``.
    """
    if len(prix) == 0:
        return

    if len(prix) == 1:
        if total % prix[0] == 0:
            yield [int(total // prix[0])]

    for nombre in range(int((total // prix[0])+1)):
        for solution in aperitif(
                total - prix[0]*nombre,
                prix[1:]
            ):
            yield [nombre] + solution

def _chaines_prix(prix):
    """Retourne des chaines de cararctères correspondant aux prix."""
    return [str(float(item)/100) for item in prix]

def _afficher_prix(prix):
    """Affiche la liste des prix, sous forme de tableau."""
    chaines = _chaines_prix(prix)
    longueur = max([len(item) for item in chaines])
    return " | ".join([
        "{{:^{longueur}}}".format(longueur=longueur).format(item)
        for item
        in chaines
        ])

def _afficher_solution(prix, nombre, solution):
    """Affiche une solution, sous forme de tableau.

    Arguments :
    - prix : liste des prix ;
    - nombre : numéro de la solution ;
    - solution : la solution à afficher.
    """
    longueur = max([len(item) for item in _chaines_prix(prix)])
    return "Solution {:^3} : ".format(nombre) + " | ".join([
        "{{:^{longueur}}}".format(longueur=longueur).format(item)
        for item
        in solution
        ])

def boucle():
    """Boucle principale"""
    prix = _lire_prix()
    if prix:
        while True:
            total = _lire_total()
            if total is None:
                break
            print()
            print((" "*len("Solution     : ")) + _afficher_prix(prix))
            nombre = 0
            for solution in aperitif(total, prix):
                nombre += 1
                print(_afficher_solution(prix, nombre, solution))
            if nombre == 0:
                print("*Pas de solutions...*")

def main():
    """Fonction principale."""
    analyseur(VERSION).parse_args(sys.argv[1:])
    boucle()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Tests de fonctions internes"""

import unittest
import textwrap

from jouets import dobble

# pylint: disable=too-many-public-methods
class TestCore(unittest.TestCase):
    """Test de fonctions internes"""

    def test_iterateurs(self):
        """Itérateurs"""
        ensemble_cartes = {
            dobble.CarteDobble(['1', '2', '3']),
            dobble.CarteDobble(['1', 'a', 'b']),
            dobble.CarteDobble(['1', 'x', 'y']),
            dobble.CarteDobble(['2', 'a', 'x']),
            dobble.CarteDobble(['2', 'b', 'y']),
            dobble.CarteDobble(['3', 'a', 'y']),
            dobble.CarteDobble(['3', 'b', 'x']),
            }
        jeu = dobble.JeuDobble(ensemble_cartes)

        for carte in jeu:
            self.assertIn(carte, ensemble_cartes)

        for symbol in jeu.cartes[0]:
            self.assertIn(symbol, ['1', '2', '3', 'a', 'b', 'x', 'y'])

    def test_str(self):
        """Conversion en chaînes de caractères"""
        jeu = dobble.JeuDobble({
            dobble.CarteDobble(['1', '2', '3']),
            dobble.CarteDobble(['1', 'a', 'b']),
            dobble.CarteDobble(['1', 'x', 'y']),
            dobble.CarteDobble(['2', 'a', 'x']),
            dobble.CarteDobble(['2', 'b', 'y']),
            dobble.CarteDobble(['3', 'a', 'y']),
            dobble.CarteDobble(['3', 'b', 'x']),
            })

        self.assertEqual(
            str(dobble.CarteDobble(['1', 'a', 'bc'])),
            "1 a bc"
            )

        self.assertEqual(
            str(jeu),
            textwrap.dedent("""\
                1 2 3
                1 a b
                1 x y
                2 a x
                2 b y
                3 a y
                3 b x""")
            )

    def test_equal(self):
        """Test d'égalité"""
        self.assertNotEqual(
            dobble.CarteDobble(['1', '2']),
            dobble.CarteDobble(['12'])
            )
        self.assertEqual(
            dobble.CarteDobble(['1', '2']),
            dobble.CarteDobble(['2', '1'])
            )

        self.assertNotEqual(
            dobble.JeuDobble({
                dobble.CarteDobble(['1', '2']),
                dobble.CarteDobble(['2', '3'])
                }),
            dobble.JeuDobble({
                dobble.CarteDobble(['1', '2']),
                dobble.CarteDobble(['23'])
                })
            )
        self.assertEqual(
            dobble.JeuDobble({
                dobble.CarteDobble(['1', '2']),
                dobble.CarteDobble(['2', '3'])
                }),
            dobble.JeuDobble({
                dobble.CarteDobble(['3', '2']),
                dobble.CarteDobble(['2', '1'])
                })
            )

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2012-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Lecture de fichiers représentants des grilles à compléter"""

import math
import textwrap

def grille_vierge(taille, init=None, *args, **kwargs):
    """Renvoit une grille vierge de `taille`×`taille`.

    Si `init` est une fonction, la grille est initialisée avec les valeurs
    `init(*args, **kwargs)` ; sinon, elle est initialisée avec init.
    """
    def valeur_initiale():
        """Renvoit la valeur initiale"""
        if callable(init):
            return init(*args, **kwargs) #pylint: disable=star-args
        else:
            return init
    return [
        [
            valeur_initiale()
            for __inutile__ in range(taille)
        ]
        for __inutile__ in range(taille)
        ]

def charge_fichier_court(fichier):
    #pylint: disable=too-many-branches
    """Lit le fichier, et renvoit une grille (liste de listes) correspondante.

    Le fichier est au format suivant. Les espaces sont ignores. Les retours
    a la ligne sont interdits. Les "." designent des inconnus ::

        ...232....4....1
    """
    grille = None
    taille = None
    for line in fichier:
        if line.isspace():
            continue
        if taille is not None:
            raise Exception(
                "Seule une ligne du fichier peut ne pas etre vide."
                )
        nettoyee = line.strip().replace(" ", "")
        taille = int(math.sqrt(len(nettoyee)))
        grille = grille_vierge(taille)
        x, y = 0, 0
        for char in nettoyee:
            if y == taille**2:
                raise Exception("Trop de caracteres sur la ligne.")
            if char == ".":
                pass
            elif char.isdigit() or (char.isalnum() and char.islower()):
                if char.isdigit():
                    valeur = int(char) - 1
                elif char.isalpha() and char.islower():
                    valeur = ord(char) - 88
                if 0 <= valeur < taille**2:
                    grille[x][y] = valeur
                else:
                    raise Exception(textwrap.dedent("""\
                            Nombre %s trop grand pour la taille de la
                            grille.
                            """ % char))
            else:
                raise Exception(textwrap.dedent("""\
                    Caractere %s interdit. Seuls les chiffres et les lettre
                    minuscules de 'a' a 'z' sont autorisees.
                    """ % char))
            x += 1
            if x == taille:
                x = 0
                y += 1
    return grille

def charge_fichier_long(fichier):
    """Lit le fichier, et renvoit une grille (liste de listes) correspondante.

    Le fichier est au format suivant. Les retours a la ligne sont
    significatifs. Les "." designent des inconnus ::

        . . . 2
        3 2 . .
        . . 4 .
        . . . 1
    """
    lignes = fichier.readlines()
    taille = len(lignes[0].strip().split(" "))
    grille = grille_vierge(taille)
    y = 0
    for line in lignes:
        x = 0
        for word in line.strip().split(" "):
            if word == "":
                continue
            elif x == taille:
                raise Exception(textwrap.dedent("""\
                        Ligne %s trop longue. Je m'attends a des lignes de
                        %s cases.
                        """ % (y+1, taille)))
            elif word == ".":
                pass
            elif not word.isdigit():
                raise Exception(textwrap.dedent("""\
                        La chaine %s (ligne %s) n'est ni un nombre, ni
                        '.'.
                        """ % (word, y+1)))
            else:
                grille[x][y] = int(word) - 1
            x += 1
        if x != taille:
            raise Exception(textwrap.dedent("""\
                Ligne %s trop courte. Je m'attends a des lignes de %s
                cases.
                """ % (y+1, taille)))
        y += 1
        if y == taille:
            break
    return grille

def charge_fichier(fichier, format_fichier):
    """Lit le fichier, et renvoit une liste de listes correspondant.

    :param file fichier: Fichier à lire
    :param str format_fichier: Format du fichier, qui est "short" ou "long".
    """
    if format_fichier == "short":
        return charge_fichier_court(fichier)
    else:
        return charge_fichier_long(fichier)



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Outils pour l'analyse des arguments de ligne de commande"""

import argparse
import logging

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.StreamHandler())

def yesno(texte):
    """Interprète un texte comme un booléen"""
    return texte.lower() in ["y", "yes", "1"]

def analyseur(version, *args, **kwargs):
    """Renvoie un analyseur syntaxique

    Cet analyseur a l'option `--version`.
    """
    parser = argparse.ArgumentParser(*args, **kwargs)
    parser.add_argument(
        '-v', '--version',
        action='version',
        version='%(prog)s {version}'.format(version=version),
        )
    return parser



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Fonctions mathématiques diverses."""

import math

def est_pair(n):
    """Renvoit vrai ssi le nombre est pair.

    >>> est_pair(1)
    False
    >>> est_pair(2)
    True
    >>> est_pair(3)
    False
    >>> est_pair(4)
    True
    """
    # pylint: disable=invalid-name
    return math.floor(n/2)*2 == n

def est_carre(n):
    """Renvoit vrai ssi le nombre est un carré parfait.

    >>> est_carre(1)
    True
    >>> est_carre(2)
    False
    >>> est_carre(3)
    False
    >>> est_carre(4)
    True
    """
    # pylint: disable=invalid-name
    return math.floor(math.sqrt(n))**2 == n




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de aperitif.py"""

import unittest

from jouets.aperitif import aperitif

TEMOINS = [
    (
        # http://xkcd.com/287/
        [215, 275, 335, 355, 420, 580],
        1505,
        [[1, 0, 0, 2, 0, 1], [7, 0, 0, 0, 0, 0]],
    ),
    (
        [7, 8, 9],
        30,
        [[2, 2, 0], [3, 0, 1]],
    ),
    ]
# pylint: disable=too-many-public-methods
class TestPremiers(unittest.TestCase):
    """Vérification de solutions"""

    def test(self):
        """Quelques solutions"""
        for (prix, total, solutions) in TEMOINS:
            self.assertCountEqual(
                solutions,
                aperitif(total, prix),
            )

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Création et vérification de jeux de cartes de Dobble"""

import argparse
import functools
import itertools
import sys
import textwrap

from jouets.erathostene import est_premier
from jouets.utils.argparse import analyseur
from jouets.utils.plugins import get_plugin

VERSION = "0.1.0"

@functools.total_ordering
class Carte:
    """Une liste de symboles"""
    # pylint: disable=too-few-public-methods

    def __init__(self, symboles=None, groupe=None):
        self.groupe = groupe
        if symboles is None:
            self.symboles = []
        else:
            self.symboles = list(symboles)

    def __iter__(self):
        for symbol in self.symboles:
            yield symbol

    def __str__(self):
        return " ".join(sorted([str(s) for s in self.symboles]))

    def __lt__(self, other):
        if not isinstance(other, Carte):
            raise TypeError()
        return sorted(self.symboles) < sorted(other.symboles)

    def __eq__(self, other):
        if not isinstance(other, Carte):
            raise TypeError()
        return sorted(self.symboles) == sorted(other.symboles)

    def __hash__(self):
        return hash(str(self))

    def __len__(self):
        return len(self.symboles)

class CarteDobble(Carte):
    """Carte de jeu de Dobble."""
    @property
    def valide(self):
        """Renvoie `True` ssi la carte est valide.

        Une carte est valide si elle ne contient pas de symbole en double.
        """
        return len(set(self.symboles)) == len(self.symboles)

    def est_compatible(self, carte):
        """Renvoie `True` si `self` est compatible avec `carte`.

        Deux cartes sont compatibles si elles ont exactement un symbole en
        commun.
        """
        return len(set(self.symboles) & set(carte.symboles)) == 1

class Jeu:
    """Une liste de cartes"""
    # pylint: disable=too-few-public-methods

    def __init__(self, cartes=None):
        if cartes is None:
            self.cartes = []
        else:
            self.cartes = list(cartes)

    def __iter__(self):
        for carte in self.cartes:
            yield carte

    def __str__(self):
        return "\n".join([str(carte) for carte in sorted(self.cartes)])

    def __eq__(self, other):
        if not isinstance(other, Jeu):
            raise TypeError()
        return sorted(self.cartes) == sorted(other.cartes)

    def __hash__(self):
        return hash(" ".join([str(hash(carte)) for carte in sorted(self)]))

    @property
    def symboles(self):
        """Renvoie l'ensemble des symboles du jeu."""
        symboles = set()
        for carte in self:
            symboles |= set(carte.symboles)
        return symboles

class JeuDobble(Jeu):
    """Jeu de Dobble"""

    def __init__(self, cartes=None):
        super().__init__(cartes)
        self._resume = {
            '_hash': None,
            'valide': None,
            'trivial': None,
            'regulier': None,
            'frequences_symboles': {},
            }
    def _calcule_resume(self):
        """Met à jour, si nécessaire, le résumé du jeu.

        Le « résumé » du jeu est :
        - ses propriétés (valide, trivial, régulier) ;
        - la fréquence d'apparition des symboles.
        """
        if hash(self) != self._resume['_hash']:
            self._resume['_hash'] = hash(self)

        self._resume['valide'] = \
                (not self.cartes_invalides()) and \
                (not self.couples_cartes_invalides())

        self._resume['trivial'] = (
            # Toutes les cartes ne contiennent qu'un symbole
            {len(carte) for carte in self.cartes} == {1}
            and
            # Le jeu est valide
            self._resume['valide']
            )

        frequences = {}
        for carte in self.cartes:
            for symbol in carte:
                frequences[symbol] = frequences.get(symbol, 0) + 1
        self._resume['frequences_symboles'] = frequences

        couples_symboles = [
            (symbole1, symbole2)
            for (symbole1, symbole2)
            in itertools.combinations(frequences.keys(), 2)
            ]
        self._resume['regulier'] = (
            self._resume['valide']
            and
            # Toutes les cartes ont la même taille
            len({len(carte) for carte in self.cartes}) == 1
            and
            # Les symboles apparaissent autant de fois
            len(set(self._resume['frequences_symboles'].values())) == 1
            and
            # Deux symboles quelconques apparaissent sur exactement une carte
            (
                len(frequences) == 1
                or
                set([
                    len([
                        carte
                        for carte in self.cartes
                        if (couple[0] in carte and couple[1] in carte)
                    ])
                    for couple in couples_symboles
                ]) == {1}
                )
            )


    @property
    def valide(self):
        """Retourne `True` ssi le jeu est valide.

        Calcule à nouveau le résumé du jeu si nécessaire.
        """
        self._calcule_resume()
        return self._resume['valide']

    @property
    def regulier(self):
        """Retourne `True` ssi le jeu est régulier.

        Calcule à nouveau le résumé du jeu si nécessaire.
        """
        self._calcule_resume()
        return self._resume['regulier']

    @property
    def trivial(self):
        """Retourne `True` ssi le jeu est trivial.

        Calcule à nouveau le résumé du jeu si nécessaire.
        """
        self._calcule_resume()
        return self._resume['trivial']

    @property
    def frequences_symboles(self):
        """Retourne un dictionnaire des fréquences des symboles.

        Calcule à nouveau le résumé du jeu si nécessaire.

        Les clefs du dictionnaire sont les symboles, et les valeurs sont le
        nombre d'apparition de la clef dans le je.u.
        """
        self._calcule_resume()
        return self._resume['frequences_symboles']

    def cartes_invalides(self):
        """Retourne la liste des cartes invalides."""
        return [carte for carte in self if not carte.valide]

    def couples_cartes_invalides(self):
        """Retourne la liste des couples de cartes incompatibles."""
        return [(card1, card2)
                for (card1, card2) in itertools.combinations(self.cartes, 2)
                if not card1.est_compatible(card2)]

class TailleNonGeree(Exception):
    """Le programme ne sait pas générer de jeu à cette taille."""
    def __init__(self, taille):
        super().__init__()
        self.taille = taille

    def __str__(self):
        return "Size must be a prime number, or 1 ({} is not).".format(
            self.taille
            )

class ErreurSymboles(Exception):
    """Erreur dans la définition des symboles à utiliser."""
    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return self.message

def genere_jeu(taille):
    """Crée et retourne un jeu.

    :param int taille: Taille du jeu.
    :return: Un jeu.
    :rtype: :class:`Jeu`
    """
    # Est-ce que je sais générer un jeu de cette taille ?
    if taille != 1 and not est_premier(taille):
        raise TailleNonGeree(taille)

    # Création des `taille×taille` cartes, réparties en `taille` tas de
    # `taille` cartes.
    cartes = [
        [CarteDobble() for x in range(taille)]
        for y in range(taille)
        ]

    dernier = 0

    # Affectation des marqueurs de tas
    marqueurs = list(range(1, taille+1))
    for tas in range(taille):
        for carte in cartes[tas]:
            carte.symboles.append(marqueurs[tas])

    # Affectation des autres symboles des tas (sauf le dernier)
    symboles = [
        list(range(i * taille + 1, (i+1)*taille + 1))
        for i
        in range(1, taille+1)
        ]
    for x in range(taille):
        for y in range(taille):
            for z in range(taille):
                cartes[x][y].symboles.append(symboles[z][(x*z+y) % taille])

    # Créations des cartes de familles de symboles
    cartes.append([
        CarteDobble(famille + [dernier])
        for famille in [marqueurs] + symboles
    ])

    # Création du jeu à partir des cartes
    return JeuDobble(itertools.chain.from_iterable(cartes))


def command_check(arguments):
    """Gestion de la sous-commande 'check'.

    :param arguments: `namespace` renvoyé par :func:`argparse.parse`.
    :return: Le *status code* à retourner par le programme.
    """
    jeu = analyse_fichier(arguments.file)
    if not arguments.show:
        arguments.show = ['valid']
    if not arguments.quiet:

        if 'summary' in arguments.show:
            arguments.show.extend(['trivial', 'regular', 'valid'])
            summary = ""
            summary += "Symbols ({cardinal})".format(
                cardinal=len(jeu.frequences_symboles.keys())
                )
            summary += "".join([
                "\n\t{key}: {value}".format(key=key, value=value)
                for (key, value) in sorted(jeu.frequences_symboles.items())
                ])
            summary += "\n"
            summary += "Cards ({cardinal})".format(cardinal=len(jeu.cartes))
            summary += "".join(["\n\t" + str(carte) for carte in sorted(jeu)])
            summary += "\n"
            cartes_invalides = jeu.cartes_invalides()
            summary += "Invalid cards ({cardinal})".format(
                cardinal=len(cartes_invalides)
                )
            summary += "".join([
                "\n\t" + str(carte)
                for carte in sorted(cartes_invalides)
                ])
            summary += "\n"
            couples_cartes_invalides = jeu.couples_cartes_invalides()
            summary += "Invalid card couples ({cardinal})".format(
                cardinal=len(couples_cartes_invalides)
                )
            summary += "".join([
                "\n\t" + str(card1) + " | " + str(card2)
                for (card1, card2) in sorted(couples_cartes_invalides)
                ])
            print(summary)

    quiet_status = True
    for (check, value) in [
            ('valid', jeu.valide),
            ('regular', jeu.regulier),
            ('trivial', jeu.trivial),
        ]:
        if check in arguments.show:
            quiet_status = quiet_status and value
            print("{}: ".format(check))
            if value:
                print('yes')
            else:
                print('no')

    if not arguments.quiet:
        return 0
    if quiet_status:
        return 0
    else:
        return 1

def analyse_fichier(fileobject):
    """Analyse le fichier donné.

    :param file fileobject: Fichier à analyser.
    :return: Un jeu.
    :rtype: :class:`Jeu`
    """
    jeu = JeuDobble()
    for ligne in fileobject:
        if ligne.strip():
            jeu.cartes.append(CarteDobble(ligne.split()))
    return jeu

def analyse():
    """Renvoie un analyseur de ligne de commande."""
    parser = analyseur(VERSION)
    subparsers = parser.add_subparsers(title='Commands', dest='command')
    subparsers.required = True
    subparsers.dest = 'command'

    # Check
    check = subparsers.add_parser(
        'check',
        help='Check properties about the game (default is validity).',
        description='Check properties about the game.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent("""\
                Syntax
                  Games are given line per line: each line is a different
                  card (blank lines are ignored). Each card is a white
                  space separated list of symbols (a symbol can be any
                  string not containing white spaces).
                """),
        )
    check.add_argument(
        '-f', '--file',
        nargs='?', type=argparse.FileType('r'), default=sys.stdin,
        help='Input file. Default is standard input.'
        )
    check.add_argument(
        '-r', '--regular',
        dest='show', action='append_const', const='regular',
        help=('Check if game is regular.')
        )
    check.add_argument(
        '-v', '--valid',
        dest='show', action='append_const', const='valid',
        help=('Check if game is valid.'),
        )
    check.add_argument(
        '-t', '--trivial',
        dest='show', action='append_const', const='trivial',
        help=('Check if game is trivial.'),
        )
    check.add_argument(
        '-s', '--summary',
        dest='show', action='append_const', const='summary',
        default=False,
        help='Print somme information about the game.'
        )
    check.add_argument(
        '-q', '--quiet',
        action='store_true',
        help=(
            "Does not print anything: exit status is 0 if condition are met, "
            "1 otherwise."
            )
        )

    # Build
    build = subparsers.add_parser(
        'build',
        help='Build some game.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    build.add_argument(
        '-s', '--size',
        dest='size', action='store', type=int,
        required=True,
        help=('Size of the game.')
        )

    output_plugins = get_plugin('dobble', 'output')
    build.add_argument(
        '-f', '--format',
        choices=output_plugins.keys(),
        default='raw',
        help=(
            "Output format: {}.".format(
                ", ".join([
                    "'{}' ({})".format(keyword, value['description'])
                    for keyword, value
                    in output_plugins.items()
                    ])
            )
        )
        )

    return parser

def main():
    """Fonction principale"""
    # Argument parsing
    arguments = analyse().parse_args(sys.argv[1:])

    # Running subcommands
    if arguments.command == 'check':
        status = command_check(arguments)
    elif arguments.command == 'build':
        print(get_plugin("dobble", "output")[arguments.format]['genere'](
            genere_jeu(arguments.size)
            ))
        status = 0
    else:
        status = 1

    # End
    return status



#!/usr/bin/env python3

# Copyright 2012-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de la vérification de jeux de dobble."""

import unittest
import io

from jouets import dobble

IRREGULIER = """\
        1 2 3
        1 a b
        1 x y
        2 a x
        2 b y
        3 a y
        """

# pylint: disable=too-many-public-methods
class TestAnalyseFichier(unittest.TestCase):
    """Test de lecture et analyse de fichiers."""

    def test_fichier_vide(self):
        """Lecture de fichier vide."""
        self.assertEqual(
            dobble.analyse_fichier(io.StringIO('')),
            dobble.JeuDobble()
            )

    def test_fichier_valide(self):
        """Lecture de jeux valides."""
        self.assertEqual(
            dobble.analyse_fichier(io.StringIO("""\
                1 2 a z
                a b 23 t

                t 2
                """)),
            dobble.JeuDobble({
                dobble.CarteDobble(['1', '2', 'a', 'z']),
                dobble.CarteDobble(['a', 'b', '23', 't']),
                dobble.CarteDobble(['t', '2']),
                })
            )

class TestUtils(unittest.TestCase):
    """Tests de fonctions intermédiaires."""

    def test_resume(self):
        """Test du résumé."""
        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 b
                1 a
                1 2
                2 a b
                """))
        self.assertDictEqual(
            jeu.frequences_symboles,
            {
                '1': 3,
                'a': 2,
                'b': 2,
                '2': 2,
            }
            )

    def test_ensemble(self):
        """Test des fonctions ensemblistes."""
        jeu = dobble.analyse_fichier(io.StringIO(IRREGULIER))
        self.assertFalse(jeu.regulier)

        jeu.cartes.append(dobble.CarteDobble(['3', 'b', 'x']))
        self.assertTrue(jeu.regulier)

        jeu.cartes[0].symboles.append('*')
        self.assertFalse(jeu.regulier)



class TestVerification(unittest.TestCase):
    """Test de vérification de jeux"""

    def test_valide(self):
        """Validité"""
        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 2 3
                1 2 a
                1 3 b
                """))
        self.assertFalse(jeu.valide)

        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 2
                1
                """))
        self.assertTrue(jeu.valide)

        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 1
                1 2
                """))
        self.assertFalse(jeu.valide)


    def test_regulier(self):
        """Régularité"""
        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 2 3
                1 a b
                1 x y
                2 a x
                2 b y
                3 a y
                3 b x
                """))
        self.assertTrue(jeu.regulier)

        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 2
                1 3
                2 3
                1 3
                """))
        self.assertFalse(jeu.valide)
        self.assertFalse(jeu.regulier)

    def test_trivial(self):
        """Trivialité"""
        jeu = dobble.analyse_fichier(io.StringIO("""\
                1 2 3
                1 a b
                1 x y
                2 a x
                2 b y
                3 a y
                3 b x
                """))
        self.assertFalse(jeu.trivial)

        jeu = dobble.analyse_fichier(io.StringIO("""\
                1
                1
                1
                1
                1
                """))
        self.assertTrue(jeu.valide)
        self.assertTrue(jeu.trivial)
        self.assertTrue(jeu.regulier)

        jeu = dobble.analyse_fichier(io.StringIO("""\
                1
                """))
        self.assertTrue(jeu.valide)
        self.assertTrue(jeu.trivial)
        self.assertTrue(jeu.regulier)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2012-2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de la génération de jeux."""

import unittest

from jouets import dobble

# pylint: disable=too-many-public-methods
class TestBuild(unittest.TestCase):
    """Test de la génération de jeux."""

    def test_exception(self):
        """Génération incorrecte"""
        for taille in [4, 6, 100]:
            self.assertRaises(
                dobble.TailleNonGeree,
                dobble.genere_jeu, taille
                )

    def test_regulier(self):
        """Génération correcte."""
        for taille in [1, 2, 3, 5, 7, 11]:
            self.assertTrue(dobble.genere_jeu(taille).regulier)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Test de chemin.py"""

import unittest

from jouets.chemin import Resolution

TEMOIN = [
    [1, 3, 10, 2, 7, 20, 1, 30, 57],
    [1, 3, 10, 2, 7, 20, 1, 57, 27],
    [1, 30, 57, 2, 7, 20, 1, 3, 10],
    [1, 57, 27, 2, 7, 20, 1, 3, 10],
    ]

# pylint: disable=too-many-public-methods
class TestOptimales(unittest.TestCase):
    """Recherche et vérifie les solutions optimales"""

    def test(self):
        """Vérifie les solutions optimales"""
        solutions = Resolution()
        solutions.recherche()

        self.assertCountEqual(
            [solution.array.array for solution in solutions.top],
            TEMOIN,
        )

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Quelques fonctions utiles manipulant listes et objets apparentés."""

from jouets.utils.erreurs import ErreurInterne

def autre(couple, element):
    """Renvoie l'autre élément du couple.

    :param tuple couple: Couple d'objets.
    :param element: Élement.
    :raises `jouets.utils.erreurs.ErreurInterne`: Si ``element`` n'est pas un
      des deux éléments de ``couple``.
    """
    if couple[0] == element:
        return couple[1]
    elif couple[1] == element:
        return couple[0]
    raise ErreurInterne()

class ListeTriee:
    """Liste triée

    Les objets mis dans la liste sont un couple (valeur, clef). Les valeurs
    sont alors triées selon la clef. La liste est maintenue triée, donc
    obtenir le plus petit élément de la liste est fait en temps contsant.
    """

    def __init__(self):
        self.liste = []

    def push(self, valeur, clef):
        """Insère un élément dans la liste.

        L'élément est ``valeur``, trié selon ``clef``.

        :param object valeur: Élément inséré dans la liste.
        :param int clef: Clef selon laquelle sont triés les éléments.

        Le type de ``valeur`` est libre.
        """
        if not self.liste:
            self.liste.append((valeur, clef))
            return

        (minimum, maximum) = (0, len(self.liste))
        while minimum != maximum:
            moitie = (minimum + maximum) // 2
            if self.liste[moitie][1] < clef:
                minimum = moitie + 1
            else:
                maximum = moitie

        self.liste.insert(minimum, (valeur, clef))

    def pop(self):
        """Retourne (et enlève de la liste) le plus petit élément."""
        return self.liste.pop(0)

    def suivant(self, depart, pas):
        r"""Renvoie la première clef libre de la liste.

        Cette clef libre est le premier entier de la forme :math:`depart + pas
        \times n` (où :math:`n` est dans :math:`\mathbb{N}^*`), tel qu'aucun
        élément de la liste ne corresponde à cette clef.
        """
        candidat = depart
        indice = 0
        while True:
            candidat += pas
            while self.liste[indice][1] < candidat:
                indice += 1
            if self.liste[indice][1] != candidat:
                return candidat




#!/usr/bin/env python3

# Copyright 2014 Louis Paternault
#
# This file is part of Jouets.
#
# Jouets is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Jouets is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Jouets.  If not, see <http://www.gnu.org/licenses/>.

"""Labyrinthe à base d'un maillage aléatoire.

Le rendu n'est pas très utilisable. À améliorer.
"""

import random

from jouets.labyrinthe.shapes.turtle import \
        LabyrintheTurtle, MurSegment, ZoneTurtle
from jouets.utils.listes import autre

def complete_analyseur(analyseur, _path, _module, parent):
    """Ajoute des options à la ligne de commande."""
    analyseur_random1 = analyseur.add_parser(
        "random1",
        help="Random structure",
        description=(
            "Two dimension labyrinth with a random basic structure (I am not "
            "satisfied with the result, though)."
            ),
        parents=parent,
        )
    analyseur_random1.set_defaults(labyrinthe=Labyrinthe)

class Labyrinthe(LabyrintheTurtle):
    """Labyrinthe en deux dimensions à partir d'un maillage aléatoire.

    :param int taille: Taille du maillage
    :param bool affiche: Même signification que pour
        :class:`~jouets.labyrinthe.shapes.turtle.LabyrintheTurtle`.
    """

    #pylint: disable=abstract-class-little-used

    def __init__(self, taille, affiche):
        super().__init__((0, 0, 100, 100), affiche)

        # Rectangle initial
        self.murs.add(MurSegment(self, (0, 0), (0, 100)))
        self.murs.add(MurSegment(self, (0, 100), (100, 100)))
        self.murs.add(MurSegment(self, (100, 100), (100, 0)))
        self.murs.add(MurSegment(self, (100, 0), (0, 0)))
        zone = ZoneTurtle(self)
        for mur in self.murs:
            mur.affecte_zone(zone)
        self.zones = {zone}
        zone.dessine()

        # Partitions aléatoires
        for __ignored in range(taille):
            zone = max(self.zones)
            if len(zone.murs) == 3:
                (mur1, mur2) = random.sample(zone.murs, 2)
            else:
                mur1 = random.choice(list(zone.murs))
                mur2 = random.choice([
                    mur for mur in zone.murs
                    if set(mur.coordonnees).isdisjoint(set(mur1.coordonnees))
                    ])
            self.mur_aleatoire(zone, mur1, mur2)

    @staticmethod
    def point_aleatoire(alea, point1, sommet2):
        "Renvoie un point aléatoire situé sur le segment [`point1`, `point2`]"

        return tuple([
            point1[i] + alea * (sommet2[i] - point1[i]) for i in range(2)
            ])

    def mur_aleatoire(self, zone, mur1, mur2):
        """Construit un mur aléatoire

        :param zone: Zone étant partagée en deux nouvelles zones.
        :type zone: :class:`ZoneBase`.
        :param mur1: Premier mur d'où va partir le nouveau mur.
        :param mur2: Second mur où va arriver le nouveau mur.
        :type mur1: :class:`MurTurtle`
        :type mur2: :class:`MurTurtle`
        """
        (alea1, alea2) = [random.uniform(.2, .8)] * 2 #pylint: disable=unbalanced-tuple-unpacking
        point1 = self.point_aleatoire(alea1, *mur1.extremites)
        point2 = self.point_aleatoire(alea2, *mur2.extremites)

        zonebis = ZoneTurtle(self)
        self.zones.add(zonebis)

        mur1bis = MurSegment(self, mur1.extremites[0], point1)
        mur1.coordonnees = (point1, mur1.extremites[1])
        for element in mur1.zones:
            if element:
                mur1bis.affecte_zone(element)
        self.murs.add(mur1bis)

        #pylint: disable=undefined-loop-variable
        for (point, mur) in zone.iter_sommets_murs(point1, mur1bis):
            if mur == mur2:
                break
            mur.change_zone(zone, zonebis)

        mur2bis = MurSegment(self, point, point2)
        mur2bis.affecte_zone(zone)
        mur2bis.affecte_zone(autre(mur2.zones, zone))
        self.murs.add(mur2bis)

        mur2.coordonnees = [point2, autre(mur.extremites, point)]
        mur2.change_zone(zone, zonebis)

        transversal = MurSegment(self, point1, point2)
        transversal.affecte_zone(zone)
        transversal.affecte_zone(zonebis)
        transversal.dessine()
        self.murs.add(transversal)
